from unittest import TestCase as UnitTestCase
import pytest
from django.conf import settings
from django.contrib.auth import logout, login
from django.contrib.auth.models import User, Group
from django.contrib.sessions.middleware import SessionMiddleware
from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.test import RequestFactory, TestCase
try:
    from unittest.mock import Mock, patch
except:
    from mock import Mock, patch

from django_roles_access.models import ViewAccess
from django_roles_access.tools import (get_setting_dictionary, get_view_access,
                                       check_access_by_role, get_app_type,
                                       get_forbidden_message,
                                       DEFAULT_FORBIDDEN_MESSAGE,
                                       get_no_access_response)


@patch('django_roles_access.tools.resolve')
@patch('django_roles_access.tools.ViewAccess.objects')
class UnitTestGetViewAccess(UnitTestCase):

    def setUp(self):
        self.request = Mock()
        self.request.user = Mock()

    def test_filter_is_done_with_view_name(
            self, mock_objects, mock_resolve
    ):
        used_url = Mock()
        used_url.view_name = 'fake-namespace:fake-view-name'
        mock_resolve.return_value = used_url

        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = Mock()
        get_view_access(self.request)
        mock_objects.filter.assert_called_with(
            view='fake-namespace:fake-view-name')

    def test_secured_view_as_public_with_no_authorization_and_no_role(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert get_view_access(self.request)

    def test_secured_view_as_authorized_user_is_authenticated(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'au'
        self.request.user.is_authenticated = True
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert get_view_access(self.request)

    def test_secured_view_as_authorized_user_is_not_authenticated(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'au'
        self.request.user.is_authenticated = False
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert not get_view_access(self.request)

    def test_secured_view_by_role_user_is_not_authenticated(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'br'
        self.request.user.is_authenticated = False
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert not get_view_access(self.request)

    def test_secured_view_by_role_user_is_not_authenticated_not_in_group(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'br'
        view_access.roles.all.return_value = ['need-access', 'fake-access']
        self.request.user.is_authenticated = True
        self.request.user.groups.all.return_value = []
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert not get_view_access(self.request)

    def test_secured_view_by_role_user_is_authenticated_and_in_group(
            self, mock_objects, mock_resolve
    ):
        view_access = Mock()
        view_access.type = 'br'
        view_access.roles.all.return_value = ['need-access', 'fake-access']
        self.request.user.is_authenticated = True
        self.request.user.groups.all.return_value = ['fake-access']
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        assert get_view_access(self.request)
        # self.fail(u'Finish this???')

    def test_get_view_access_return_None_in_no_view_access_object(
            self, mock_objects, mock_resolve
    ):
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        self.assertEqual(get_view_access(self.request), None)


@pytest.mark.django_db
class TestGetViewAccess(UnitTestCase):

    def setUp(self):
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get('/role-included1/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

        # ViewAccess
        self.view_access, created = ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='pu'
        )

    def fixture_role(self, user, view_access):
        user.groups.add(self.g1)
        user.save()
        view_access.roles.add(self.g1)
        view_access.save()

    def test_secured_view_as_public_with_no_authorization_and_no_role(self):
        self.view_access.type = 'pu'
        self.view_access.save()
        logout(self.req1)
        assert get_view_access(self.req1)

    def test_secured_view_as_public_with_authorization_and_no_role(self):
        self.view_access.type = 'pu'
        self.view_access.save()
        login(self.req1, self.u1)
        assert get_view_access(self.req1)

    def test_secured_view_as_public_with_no_authorization_and_role(self):
        self.view_access.type = 'pu'
        self.view_access.save()
        logout(self.req1)
        assert get_view_access(self.req1)

    def test_secured_view_as_public_with_authorization_and_role(self):
        self.view_access.type = 'pu'
        self.view_access.save()
        self.fixture_role(self.u1, self.view_access)
        login(self.req1, self.u1)
        assert get_view_access(self.req1)

    def test_secured_view_as_authorized_with_no_authorization_and_no_role(self):
        self.view_access.type = 'au'
        self.view_access.save()
        logout(self.req1)
        assert not get_view_access(self.req1)

    def test_secured_view_as_authorized_with_authorization_and_no_role(self):
        self.view_access.type = 'au'
        self.view_access.save()
        login(self.req1, self.u1)
        assert get_view_access(self.req1)

    def test_secured_view_as_authorized_with_no_authorization_and_role(self):
        self.view_access.type = 'au'
        self.view_access.save()
        self.fixture_role(self.u1, self.view_access)
        logout(self.req1)
        assert not get_view_access(self.req1)

    def test_secured_view_as_authorized_with_authorization_and_role(self):
        self.view_access.type = 'au'
        self.view_access.save()
        self.fixture_role(self.u1, self.view_access)
        login(self.req1, self.u1)
        assert get_view_access(self.req1)

    def test_secured_view_by_role_with_no_authorization_and_no_role(self):
        self.view_access.type = 'br'
        self.view_access.save()
        self.view_access.roles.add(self.g1)
        self.view_access.save()
        logout(self.req1)
        assert not get_view_access(self.req1)

    def test_secured_view_by_role_with_authorization_and_no_role(self):
        self.view_access.type = 'br'
        self.view_access.save()
        self.view_access.roles.add(self.g1)
        self.view_access.save()
        self.u1.groups.clear()
        self.u1.save()
        logout(self.req1)
        login(self.req1, self.u1)
        assert not get_view_access(self.req1)

    def test_secured_view_by_role_with_no_authorization_and_role(self):
        self.view_access.type = 'br'
        self.view_access.save()
        self.fixture_role(self.u1, self.view_access)
        logout(self.req1)
        assert not get_view_access(self.req1)

    def test_secured_view_by_role_with_authorization_and_role(self):
        self.view_access.type = 'br'
        self.view_access.save()
        self.fixture_role(self.u1, self.view_access)
        login(self.req1, self.u1)
        assert get_view_access(self.req1)


@pytest.mark.django_db
class TestGetViewAccessWithDirectView(UnitTestCase):

    def setUp(self):
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get('/direct_access_view/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

        # ViewAccess
        self.view_access, created = ViewAccess.objects.get_or_create(
            view='direct_access_view',
            type='au'
        )

    def test_secured_view_without_app_name_with_authentication(self):
        login(self.req1, self.u1)
        self.assertTrue(get_view_access(self.req1))

    def test_secured_view_without_app_name_without_authentication(self):
        logout(self.req1)
        assert not get_view_access(self.req1)


@pytest.mark.django_db
class TestNestedNameSpaces(UnitTestCase):

    def setUp(self):
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get(
            '/nest1/nest2/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

        # ViewAccess
        self.view_access, created = ViewAccess.objects.get_or_create(
            view='nest1_namespace:nest2_namespace:view_protected_by_role',
            type='au'
        )

    def test_secured_view_with_nested_namespace(self):
        login(self.req1, self.u1)
        self.assertTrue(get_view_access(self.req1))

    def test_secured_view_without_nested_namespace_without_authentication(self):
        logout(self.req1)
        assert not get_view_access(self.req1)

    def test_check_secured_view_without_nested_namespace_without_authentication(
            self):
        view_access, created = ViewAccess.objects.get_or_create(
            view='direct_access_view',
            type='au'
        )
        request = RequestFactory().get('/direct_access_view/')
        request.user = self.u1
        middleware = SessionMiddleware()
        middleware.process_request(request)
        request.session.save()
        logout(request)
        assert not check_access_by_role(request)


class TestGetDictionarySettings(UnitTestCase):
    """
    """

    def test_get_dictionary_with_settings_variables_all_Nones(self):
        """
        """
        expected_dictionary = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': [],
        }
        settings_dictionary = get_setting_dictionary()
        self.assertEqual(expected_dictionary, settings_dictionary)

    def test_get_dictionary_with_settings_variables_SECURED(self):
        """
        """
        settings.__setattr__('SECURED', ['one_application'])
        expected_dictionary = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': ['one_application'],
        }
        settings_dictionary = get_setting_dictionary()
        # Clear mock value to not interfere with other tests
        settings.__delattr__('SECURED')
        self.assertEqual(expected_dictionary, settings_dictionary)

    def test_get_dictionary_with_settings_variables_NOT_SECURED(self):
        """
        """
        settings.__setattr__('NOT_SECURED', ['two_application'])
        expected_dictionary = {
            'NOT_SECURED': ['two_application'],
            'PUBLIC': [],
            'SECURED': [],
        }
        settings_dictionary = get_setting_dictionary()
        # Clear mock value to not interfere with other tests
        settings.__delattr__('NOT_SECURED')
        self.assertEqual(expected_dictionary, settings_dictionary)

    def test_get_dictionary_with_settings_variables_PUBLIC(self):
        """
        """
        settings.__setattr__('PUBLIC', ['last_application'])
        expected_dictionary = {
            'NOT_SECURED': [],
            'PUBLIC': ['last_application'],
            'SECURED': [],
        }
        settings_dictionary = get_setting_dictionary()
        # Clear mock value to not interfere with other tests
        settings.__delattr__('PUBLIC')
        self.assertEqual(expected_dictionary, settings_dictionary)

    def test_get_dictionary_with_settings_variables_with_combination(self):
        """
        """
        settings.__setattr__('PUBLIC', ['last_application'])
        settings.__setattr__('SECURED', ['one_application'])
        expected_dictionary = {
            'NOT_SECURED': [],
            'PUBLIC': ['last_application'],
            'SECURED': ['one_application'],
        }
        settings_dictionary = get_setting_dictionary()
        # Clear mock value to not interfere with other tests
        settings.__delattr__('PUBLIC')
        settings.__delattr__('SECURED')
        self.assertEqual(expected_dictionary, settings_dictionary)


@patch('django_roles_access.tools.resolve')
@patch('django_roles_access.tools.get_view_access')
class UnitTestCheckAccessByRole(UnitTestCase):

    def setUp(self):
        self.request = Mock()

    def test_resolve_called_with_request_path_info(
            self, mock_get_view_access, mock_resolve
    ):
        check_access_by_role(self.request)
        mock_resolve.assert_called_once_with(self.request.path_info)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_setting_dictionary_is_called(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        check_access_by_role(self.request)
        assert mock_get_setting_dictionary.called

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_setting_dictionary_is_called_once(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        check_access_by_role(self.request)
        self.assertEqual(mock_get_setting_dictionary.call_count, 1)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_app_name_is_searched_as_NOT_SECURED_before_get_view_access(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        """
        Default behavior of NOT_SECURED applications is not to check access.
        """
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': ['fake-app-name']
        }
        assert check_access_by_role(self.request)
        assert not mock_get_view_access.called

    def test_get_view_access_is_called(
            self, mock_get_view_access, mock_resolve
    ):
        check_access_by_role(self.request)
        assert mock_get_view_access.called

    def test_get_view_access_is_called_once(
            self, mock_get_view_access, mock_resolve
    ):
        check_access_by_role(self.request)
        self.assertEqual(mock_get_view_access.call_count, 1)

    def test_get_view_access_is_called_with(
            self, mock_get_view_access, mock_resolve
    ):
        self.request.user = Mock()
        check_access_by_role(self.request)
        mock_get_view_access.assert_called_once_with(
            self.request
        )

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_view_access_take_precedence_over_default_behavior(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        """
        If a ViewAccess object exist for the view, this must be first than
        default behaviors.
        """
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': ['fake-app-name']
        }
        mock_get_view_access.return_value = True
        self.request.user.is_authenticated = False
        assert check_access_by_role(self.request)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_default_PUBLIC_behavior(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': [],
            'PUBLIC': ['fake-app-name'],
        }
        mock_get_view_access.return_value = None
        assert check_access_by_role(self.request)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_default_SECURED_behavior_when_user_not_authenticated(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': ['fake-app-name']
        }
        mock_get_view_access.return_value = False
        self.request.user.is_authenticated = False
        assert not check_access_by_role(self.request)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_default_SECURED_behavior_when_user_authenticated(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': ['fake-app-name']
        }
        mock_get_view_access.return_value = None
        self.request.user.is_authenticated = True
        assert check_access_by_role(self.request)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_check_access_by_role_return_True_if_any_happen(
            self, mock_get_setting_dictionary, mock_get_view_access,
            mock_resolve
    ):
        """
        In case Django roles is activated, for example with middleware, and
        no more configuration are given, behavior must be: not to change view
        behavior.
        """
        used_url = Mock()
        used_url.app_name = 'fake-app-name'
        mock_resolve.return_value = used_url
        mock_get_setting_dictionary.return_value = {
            'NOT_SECURED': [],
            'PUBLIC': [],
            'SECURED': []
        }
        mock_get_view_access.return_value = None
        self.request.user.is_authenticated = False
        assert check_access_by_role(self.request)


# Integrated tests
class TestCheckAccessByRoleWithSecuredApplications(TestCase):

    def setUp(self):
        settings.__setattr__('SECURED', ['django_roles'])
        # User
        self.u1, created = User.objects.get_or_create(username='test-1')

        # Request
        self.req1 = RequestFactory().get('/role-included1/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

    def tearDown(self):
        settings.__delattr__('SECURED')

    def test_not_authorized_user_can_not_access(self):
        """
        By default SECURED applications require *authorized* access if there
        is no ViewAccess created for the view.
        """
        logout(self.req1)
        assert not check_access_by_role(self.req1)

    def test_authorized_user_can_access_views_without_configuration(self):
        """
        User is authorized, should be able to access views without ViewAccess
        created when is a SECURED application.
        :return:
        """
        login(self.req1, self.u1)
        assert check_access_by_role(self.req1)


class TestCheckAccessByRoleWithPublicApplications(TestCase):

    def setUp(self):
        settings.__setattr__('PUBLIC', ['django_roles'])
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get('/role-included1/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

    def tearDown(self):
        settings.__delattr__('PUBLIC')

    def fixture_role(self, user, view_access):
        user.groups.add(self.g1)
        user.save()
        view_access.roles.add(self.g1)
        view_access.save()

    def test_not_authenticated_user_can_access(self):
        logout(self.req1)
        assert check_access_by_role(self.req1)

    def test_authorized_user_can_access(self):
        login(self.req1, self.u1)
        assert check_access_by_role(self.req1)


class TestCheckAccessByRoleWithNotSecuredApplications(TestCase):
    """
    NOT_SECURED applications should not be taken in consideration.
    """

    def setUp(self):
        settings.__setattr__('NOT_SECURED', ['django_roles'])
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get('/role-included1/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

    def tearDown(self):
        settings.__delattr__('NOT_SECURED')

    def test_not_secured_app_views_are_ignored_without_authentication(self):
        """
        Despite NOT SECURED applications should be applications without any view
        the test will verify NOT SECURED application are really ignored.
        """
        logout(self.req1)
        assert check_access_by_role(self.req1)

    def test_not_secured_app_views_are_ignored_with_authentication(self):
        """
        Despite NOT SECURED applications should be applications without any view
        the test will verify NOT SECURED application are really ignored.
        """
        login(self.req1, self.u1)
        assert check_access_by_role(self.req1)

    def test_not_secured_app_views_have_no_precedence_against_ViewAccess(self):
        """
        Despite NOT SECURED applications should be applications without any view
        the test will verify that if exist a ViewAccess for the view being
        request by the user, this will be ignored.
        """
        ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='au')
        logout(self.req1)
        assert check_access_by_role(self.req1)


class TestCheckAccessByRoleWithoutAnySettings(TestCase):

    def setUp(self):
        # User and group
        self.u1, created = User.objects.get_or_create(username='test-1')
        self.g1, created = Group.objects.get_or_create(name='test-group-1')

        # Request
        self.req1 = RequestFactory().get('/role-included1/view_by_role/')
        self.req1.user = self.u1

        # Session
        middleware = SessionMiddleware()
        middleware.process_request(self.req1)
        self.req1.session.save()

    def test_any_user_with_authentication_can_access(self):
        login(self.req1, self.u1)
        assert check_access_by_role(self.req1)

    def test_any_user_without_authentication_can_access(self):
        logout(self.req1)
        assert check_access_by_role(self.req1)

    def test_ViewAccess_take_precedence_over_no_configuration_no_login(self):
        ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='au')
        logout(self.req1)
        assert not check_access_by_role(self.req1)

    def test_ViewAccess_take_precedence_over_no_configuration_with_login(self):
        ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='au')
        login(self.req1, self.u1)
        # import pdb
        # pdb.set_trace()
        assert check_access_by_role(self.req1)

    def test_ViewAccess_denied_access_if_no_role(self):
        view_access, created = ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='br')
        view_access.roles.add(self.g1)

        login(self.req1, self.u1)
        assert not check_access_by_role(self.req1)

    def test_ViewAccess_grant_access_if_role(self):
        view_access, created = ViewAccess.objects.get_or_create(
            view='django_roles:view_protected_by_role',
            type='br')
        view_access.roles.add(self.g1)
        view_access.save()
        self.u1.groups.add(self.g1)
        self.u1.save()

        login(self.req1, self.u1)
        assert check_access_by_role(self.req1)


class UnitTestGetAppType(UnitTestCase):

    def test_detect_app_has_no_type(self):
        """
        When no configuration is given, or default case.
        """
        result = get_app_type('fake-app')
        self.assertEqual(result, None)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_settings_dictionary_is_called(
            self, mock_get_settings_dictionary
    ):
        get_app_type('fake-app')
        assert mock_get_settings_dictionary.called

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_settings_dictionary_is_called_once(
            self, mock_get_settings_dictionary
    ):
        mock_get_settings_dictionary.return_value = {'type 1': [],
                                                     'type 2': []}
        get_app_type('fake-app')
        self.assertEqual(mock_get_settings_dictionary.call_count, 1)

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_get_settings_dictionary_is_called_once_with_no_param(
            self, mock_get_settings_dictionary
    ):
        get_app_type('fake-app')
        mock_get_settings_dictionary.assert_called_once_with()

    @patch('django_roles_access.tools.get_setting_dictionary')
    def test_detect_app_is_any_configured_type(
            self, mock_get_settings_dictionary
    ):
        _dict = {
            'type 1': ['blue', 'red'],
            'any type': ['orange', 'fake-app', 'sky']
        }
        mock_get_settings_dictionary.return_value = _dict
        result = get_app_type('fake-app')
        expected = u'any type'
        self.assertEqual(result, expected)


class IntegratedTestGetAppType(UnitTestCase):

    def setUp(self):
        settings.__setattr__('NOT_SECURED', ['not_secured_app'])
        settings.__setattr__('PUBLIC', ['public_app'])
        settings.__setattr__('SECURED', ['secured_app'])

    def tearDown(self):
        settings.__delattr__('NOT_SECURED')
        settings.__delattr__('PUBLIC')
        settings.__delattr__('SECURED')

    def test_default_case_with_no_configuration(self):
        result = get_app_type('app_name')
        self.assertIs(result, None)

    def test_detect_NOT_SECURED_configured_app(self):
        result = get_app_type('not_secured_app')
        expected = 'NOT_SECURED'
        self.assertEqual(result, expected)

    def test_detect_PUBLIC_configured_app(self):
        result = get_app_type('public_app')
        expected = 'PUBLIC'
        self.assertEqual(result, expected)

    def test_detect_SECURED_configured_app(self):
        result = get_app_type('secured_app')
        expected = 'SECURED'
        self.assertEqual(result, expected)


class UnitTestGetForbiddenMessage(UnitTestCase):

    def test_default_forbidden_message(self):
        assert DEFAULT_FORBIDDEN_MESSAGE == get_forbidden_message()

    @patch('django_roles_access.tools.settings')
    def test_forbidden_configured_message(
            self, mock_settings
    ):
        mock_settings.DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE = 'fake-message'
        assert 'fake-message' == get_forbidden_message()


class IntegratedTestGetForbiddenMessage(TestCase):

    def test_default_forbidden_message(self):
        assert DEFAULT_FORBIDDEN_MESSAGE == get_forbidden_message()

    def test_forbidden_configured_message(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE',
                             'forbidden-message')
        response = get_forbidden_message()
        settings.__delattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE')
        assert 'forbidden-message' == response


class UnitTestGetNoAccessResponse(UnitTestCase):

    @patch('django_roles_access.tools.HttpResponseForbidden')
    def test_default_behavior_http_response_forbidden(
            self, mock_http_response_forbidden
    ):
        mock_http_response_forbidden.return_value = 'fake-forbidden'
        response = get_no_access_response()
        assert response == 'fake-forbidden'

    @patch('django_roles_access.tools.HttpResponseForbidden')
    def test_default_behavior_http_response_forbidden_403_Forbidden(
            self, mock_http_response_forbidden
    ):
        argument = u'<h1>403 Forbidden</h1>'
        get_no_access_response()
        mock_http_response_forbidden.assert_called_with(argument)

    @patch('django_roles_access.tools.settings')
    @patch('django_roles_access.tools.HttpResponseForbidden')
    def test_http_response_forbidden_with_configuration(
            self, mock_http_response_forbidden, mock_settings
    ):
        mock_settings.DJANGO_ROLES_ACCESS_REDIRECT = False
        mock_settings.DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE = 'fake-message'
        get_no_access_response()
        mock_http_response_forbidden.assert_called_with('fake-message')

    @patch('django_roles_access.tools.settings')
    @patch('django_roles_access.tools.HttpResponseRedirect')
    def test_redirect_if_redirect_is_configured(
            self, mock_http_response_redirect, mock_settings
    ):
        mock_settings.DJANGO_ROLES_ACCESS_REDIRECT = True
        mock_http_response_redirect.return_value = 'fake-redirect'
        response = get_no_access_response()
        assert response == 'fake-redirect'

    @patch('django_roles_access.tools.settings')
    @patch('django_roles_access.tools.HttpResponseRedirect')
    def test_redirect_if_redirect_to_LOGIN_URL(
            self, mock_http_response_redirect, mock_settings
    ):
        mock_settings.DJANGO_ROLES_ACCESS_REDIRECT = True
        mock_settings.LOGIN_URL = 'fake-login'
        get_no_access_response()
        mock_http_response_redirect.assert_called_with('fake-login')


class IntegratedTextGetNoAccessResponse(TestCase):

    def test_default_behavior_http_response_forbidden(self):
        response = get_no_access_response()
        self.assertIsInstance(response, HttpResponseForbidden)

    def test_default_behavior_http_response_forbidden_403_Forbidden(self):
        response = get_no_access_response()
        self.assertIn(DEFAULT_FORBIDDEN_MESSAGE,
                      response.content.decode('utf-8'))

    def test_http_response_forbidden_with_configuration(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE',
                             'forbidden-message')
        response = get_no_access_response()
        settings.__delattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE')
        self.assertIn('forbidden-message', response.content.decode('utf-8'))

    def test_redirect_if_redirect_is_configured(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_REDIRECT', True)
        response = get_no_access_response()
        settings.__delattr__('DJANGO_ROLES_ACCESS_REDIRECT')
        self.assertIsInstance(response, HttpResponseRedirect)

    def test_redirect_if_redirect_to_LOGIN_URL(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_REDIRECT', True)
        response = get_no_access_response()
        settings.__delattr__('DJANGO_ROLES_ACCESS_REDIRECT')
        self.assertEqual(settings.LOGIN_URL, response.url)



from django.conf import settings
from django.http import HttpResponseForbidden, HttpResponseRedirect
try:
    from django.utils.translation import gettext as _
except:
    from django.utils.translation import ugettex as _
from django.urls import resolve

from django_roles_access.models import ViewAccess

DEFAULT_FORBIDDEN_MESSAGE = _(u'<h1>403 Forbidden</h1>')


def get_view_access(request):
    """
    Check access if exist a ViewAccess object for the view being processed.

    A ViewAccess object is linked to present view when current namespace and
    view name are the value of *view* attribute of the ViewAccess object. Can
    be only one ViewAccess object for each namespace and view name. If no
    namespace is used, application name is used instead as Django default
    behavior. This also means for Django < 2.0 the attribute app_name is
    required in URLConf (Django > 2.0 has this requirement).

    If exist a ViewAccess object linked to present view, security checks are
    done to conclude if request user have access or not. In case request user
    do not have access, PermissionDenied is raised.

    :return: True if user have access. Or raise PermissionDenied.
    """
    user = request.user
    current_url = resolve(request.path_info)
    view_name = current_url.view_name

    view_access = ViewAccess.objects.filter(view=view_name).first()
    if view_access:
        if view_access.type == 'pu':
            return True
        elif view_access.type == 'au':
            if user.is_authenticated:
                return True
            else:
                return False
        elif view_access.type == 'br':
            if user.is_authenticated:
                access = list(set(user.groups.all()) &
                              set(view_access.roles.all()))
                if access:
                    return True
                else:
                    return False
            else:
                return False
        else:
            return None


def get_setting_dictionary():
    """
    Return django-roles settings variable or None.

    :return: Dictionary with three keys: NOT_SECURE, PUBLIC, SECURE with their
             settings values or None.
    """
    settings_dictionary = {}

    try:
        settings_dictionary['NOT_SECURED'] = settings.NOT_SECURED
    except AttributeError:
        settings_dictionary['NOT_SECURED'] = []

    try:
        settings_dictionary['PUBLIC'] = settings.PUBLIC
    except AttributeError:
        settings_dictionary['PUBLIC'] = []

    try:
        settings_dictionary['SECURED'] = settings.SECURED
    except AttributeError:
        settings_dictionary['SECURED'] = []

    return settings_dictionary


def check_access_by_role(request):
    """
    Given a request to access a view the function check if user (logged or
    not) can access required view.

    :param request: :class:`django.http.HttpRequest`
    :return: True if can access the view. False in other case.
    """
    current_url = resolve(request.path_info)
    app_name = current_url.app_name
    setting_dictionary = get_setting_dictionary()

    # NOT_SECURED applications are ignored
    if app_name in setting_dictionary['NOT_SECURED']:
        return True
    # If view has an access configuration, this takes precedence over
    # the classification of the application
    view_access = get_view_access(request)
    if view_access is not None:
        return view_access
    # Check for public applications
    if app_name in setting_dictionary['PUBLIC']:
        return True
    if app_name in setting_dictionary['SECURED']:
        if not request.user.is_authenticated:
            return False
    return True


def get_app_type(app_name):
    for key, val in get_setting_dictionary().items():
        if app_name in val:
            return key
    return None


def get_forbidden_message():
    if hasattr(settings, 'DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE'):
        return settings.DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE
    return DEFAULT_FORBIDDEN_MESSAGE


def get_no_access_response():
    if hasattr(settings, 'DJANGO_ROLES_ACCESS_REDIRECT'):
        if settings.DJANGO_ROLES_ACCESS_REDIRECT:
            return HttpResponseRedirect(settings.LOGIN_URL)
    if hasattr(settings, 'DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE'):
        return HttpResponseForbidden(
            settings.DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE)
    return HttpResponseForbidden(DEFAULT_FORBIDDEN_MESSAGE)



from django.conf import settings
from django.contrib.auth.models import Group
try:
    from django.utils.six import StringIO
except:
    from io import StringIO

from django_roles_access.models import ViewAccess

try:
    from unittest.mock import Mock, patch, MagicMock, ANY
except:
    from mock import Mock, patch, ANY
from unittest.case import TestCase as UnitTestCase

from django.core.management import call_command
from django.test import TestCase, modify_settings
from django_roles_access.utils import (NONE_TYPE_DEFAULT, NOT_SECURED_DEFAULT,
                                       SECURED_DEFAULT, PUBLIC_DEFAULT,
                                       APP_NAME_FOR_NONE)


@patch('django_roles_access.management.commands.checkviewaccess.import_module')
@patch('django_roles_access.management.commands.checkviewaccess.settings')
class UnitTestCheckViewAccess(UnitTestCase):

    def setUp(self):
        self.root_urlconf = Mock()

    def test_write_at_beginning_of_command_execution(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'Start checking views access.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    def test_write_when_finish_command_execution(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'End checking view access.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    def test_write_at_beginning_of_gathering_information_phase(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'Start gathering information.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    def test_import_module_is_called(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        assert mock_import_module.called

    def test_import_module_is_called_once(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        self.assertEqual(mock_import_module.call_count, 1)

    def test_import_module_is_called_once_with(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        mock_import_module.assert_called_once_with(self.root_urlconf)

    @patch('django_roles_access.management.commands.checkviewaccess.walk_site_url')
    def test_walk_site_url_is_called(
            self, mock_walk_site_url, mock_settings, mock_import_module
    ):
        mock_import_module.urlpatterns = 'fake-url-pattern'
        call_command('checkviewaccess')
        assert mock_walk_site_url.called

    @patch('django_roles_access.management.commands.checkviewaccess.walk_site_url')
    def test_walk_site_url_is_called_once(
            self, mock_walk_site_url, mock_settings, mock_import_module
    ):
        mock_import_module.urlpatterns = 'fake-url-pattern'
        call_command('checkviewaccess')
        self.assertEqual(mock_walk_site_url.call_count, 1)

    @patch('django_roles_access.management.commands.checkviewaccess.walk_site_url')
    def test_walk_site_url_is_called_once_with(
            self, mock_walk_site_url, mock_settings, mock_import_module
    ):
        urlpatterns = Mock()
        urlpatterns.urlpatterns = 'fake-urlpatterns'
        mock_import_module.return_value = urlpatterns
        call_command('checkviewaccess')
        mock_walk_site_url.assert_called_once_with('fake-urlpatterns')

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    def test_get_views_by_app_is_called(
            self, mock_get_views_by_app, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        assert mock_get_views_by_app.called

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    def test_get_views_by_app_is_called_once(
            self, mock_get_views_by_app, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        self.assertEqual(mock_get_views_by_app.call_count, 1)

    @patch('django_roles_access.management.commands.checkviewaccess.walk_site_url')
    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    def test_get_views_by_app_is_called_once_with(
            self, mock_get_views_by_app, mock_walk_site_url, mock_settings,
            mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_walk_site_url.return_value = 'fake-result'
        call_command('checkviewaccess')
        mock_get_views_by_app.assert_called_once_with('fake-result')

    def test_write_at_end_of_gathering_information_phase(
            self, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'Finish gathering information.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    def test_middleware_is_active_and_site_active_is_true(
            self, mock_settings, mock_import_module
    ):
        mock_settings.MIDDLEWARE = ['fake-middleware',
                                    'django_roles_access.middleware.RolesMiddleware',
                                    'other-fake-middleware']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'Django roles active for site: True.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    def test_middleware_is_not_active_and_site_active_is_false(
            self, mock_settings, mock_import_module
    ):
        mock_settings.MIDDLEWARE = ['fake-middleware',
                                    'other-fake-middleware']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'Django roles active for site: False.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.utils.settings')
    def test_write_at_start_of_each_application_analyze(
            self, mock_utils_settings, mock_settings, mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text1 = u'Analyzing fake-app-1:'
        expected_text2 = u'Analyzing fake-app-2:'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text1, out.getvalue())
        self.assertIn(expected_text2, out.getvalue())

    @patch('django_roles_access.utils.settings')
    def test_write_at_end_of_each_application_analyze(
            self, mock_utils_settings, mock_settings, mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text1 = u'Finish analyzing fake-app-1.'
        expected_text2 = u'Finish analyzing fake-app-2.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text1, out.getvalue())
        self.assertIn(expected_text2, out.getvalue())

    @patch('django_roles_access.utils.settings')
    def test_detect_installed_application_is_not_configured(
            self, mock_utils_settings, mock_settings, mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app-1']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'fake-app-1 has no type.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.management.commands.checkviewaccess.get_app_type')
    @patch('django_roles_access.utils.settings')
    def test_get_app_type_is_called(
            self, mock_utils_settings, mock_get_app_type, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app-1']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        assert mock_get_app_type.called

    @patch('django_roles_access.management.commands.checkviewaccess.get_app_type')
    @patch('django_roles_access.utils.settings')
    def test_get_app_type_is_called_for_each_installed_app_0(
            self, mock_utils_settings, mock_get_app_type, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = []
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        self.assertEqual(mock_get_app_type.call_count, 0)

    @patch('django_roles_access.management.commands.checkviewaccess.get_app_type')
    @patch('django_roles_access.utils.settings')
    def test_get_app_type_is_called_for_each_installed_app_3(
            self, mock_utils_settings, mock_get_app_type, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2', 'bla']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        self.assertEqual(mock_get_app_type.call_count, 3)

    @patch('django_roles_access.utils.settings')
    @patch('django_roles_access.tools.settings')
    def test_detect_installed_application_is_configured_as_NOT_SECURED(
            self, mock_tools_settings, mock_utils_settings, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app']
        mock_tools_settings.NOT_SECURED = ['fake-app']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'fake-app is NOT_SECURED type.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.utils.settings')
    @patch('django_roles_access.tools.settings')
    def test_detect_installed_application_is_configured_as_PUBLIC(
            self, mock_tools_settings, mock_utils_settings, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app']
        mock_tools_settings.PUBLIC = ['fake-app']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'fake-app is PUBLIC type.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.utils.settings')
    @patch('django_roles_access.tools.settings')
    def test_detect_installed_application_is_configured_as_SECURED(
            self, mock_tools_settings, mock_utils_settings, mock_settings,
            mock_import_module
    ):
        mock_utils_settings.INSTALLED_APPS = ['fake-app']
        mock_tools_settings.SECURED = ['fake-app']
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'fake-app is SECURED type.'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    def test_view_analyzer_is_called_0_times_when_app_have_no_views(
            self, mock_view_access_analyzer, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        call_command('checkviewaccess')
        self.assertEqual(mock_view_access_analyzer.call_count, 0)

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    def test_when_app_have_no_views_it_is_reported(
            self, mock_get_views_by_app, mock_settings, mock_import_module
    ):
        mock_get_views_by_app.return_value = {
            'fake-app-name': []
        }
        mock_settings.ROOT_URLCONF = self.root_urlconf
        out = StringIO()
        expected_text = u'\t\t{} does not have configured views.'.format(
            'fake-app-name')
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    def test_view_analyzer_is_called_3_times_when_app_have_3_views(
            self, mock_view_access_analyzer, mock_view_by_app, mock_settings,
            mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view-1'),
                                          ('/fake2/', 'fake-callback-2',
                                           'fake-view-1'),
                                          ('/fake3/', 'fake-callback-3',
                                           'fake-view-1')]}
        mock_view_access_analyzer.return_value = u'fake-analysis'
        call_command('checkviewaccess')
        self.assertEqual(mock_view_access_analyzer.call_count, 3)

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    @patch('django_roles_access.management.commands.checkviewaccess.get_app_type')
    def test_view_analyzer_is_called_1_times_with_params(
            self, mock_get_app_type, mock_view_access_analyzer,
            mock_view_by_app, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view-1')]}
        mock_get_app_type.return_value = 'fake-app-type'
        mock_view_access_analyzer.return_value = u'fake-analysis'
        call_command('checkviewaccess')
        mock_view_access_analyzer.assert_called_with('fake-app-type',
                                                     'fake-callback-1',
                                                     'fake-view-1', False)

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    def test_view_name_is_reported(
            self, mock_view_access_analyzer, mock_view_by_app, mock_settings,
            mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view')]}
        out = StringIO()
        expected_text = u'Analysis for view: fake-view'
        mock_view_access_analyzer.return_value = u'fake-analysis'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    def test_view_url_is_reported(
            self, mock_view_access_analyzer, mock_view_by_app, mock_settings,
            mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view')]}
        mock_view_access_analyzer.return_value = u'fake-analysis'
        out = StringIO()
        expected_text = u'View url: /fake1/'
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_text, out.getvalue())

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    @patch('django_roles_access.management.commands.checkviewaccess'
           '.print_view_analysis')
    def test_print_view_analyzer_is_called_1_times(
            self, mock_print_view_analysis, mock_view_access_analyzer,
            mock_view_by_app, mock_settings, mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view')]}
        mock_view_access_analyzer.return_value = u'fake-analysis'
        call_command('checkviewaccess')
        self.assertEqual(mock_print_view_analysis.call_count, 1)

    @patch('django_roles_access.management.commands.checkviewaccess.get_views_by_app')
    @patch('django_roles_access.management.commands.checkviewaccess.'
           'view_access_analyzer')
    @patch('django_roles_access.management.commands.checkviewaccess'
           '.print_view_analysis')
    def test_print_view_analyzer_is_called_1_times_with_param(
            self, mock_print_view_analysis,
            mock_view_access_analyzer, mock_view_by_app, mock_settings,
            mock_import_module
    ):
        mock_settings.ROOT_URLCONF = self.root_urlconf
        mock_view_by_app.return_value = {'fake-app':
                                         [('/fake1/', 'fake-callback-1',
                                           'fake-view')]}
        mock_view_access_analyzer.return_value = u'fake-analysis'
        call_command('checkviewaccess')
        mock_print_view_analysis.assert_called_once_with(ANY, ANY,
                                                         u'fake-analysis')


class IntegratedTestCheckViewAccess(TestCase):
    """
    Cases:
    * Test default cases: no View objects for the view but Django role tool is
      used and application has type: NOT_SECURED, PUBLIC, AUTHORIZED, By role.

    * Test no application type: no View objects for the view but Django role
      tool is used and application has no type.

    * Test no configuration: no View objects for the view, no Django role
      tool is used. and application has type and application has no type.
    """

    # RED = '\x1b[31;1m'
    # GREEN = '\x1b[32;1m'
    # WARNING = '\x1b[33;1m'
    # ATTRIBUTES_OFF = '\x1b[0m'
    NO_APP_TYPE_ERROR = u'ERROR: Django roles middleware is active; or view ' \
                        u'is protected with Django roles decorator or mixin, ' \
                        u'and has no application or application has no type. ' \
                        u'There are no View Access object for the view. Is ' \
                        u'not possible to determine behavior for access view.'

    NOT_SECURED_DEFAULT = u'WARNING: View has no security configured ' \
                          u'(ViewAccess) and application type is ' \
                          u'"NOT_SECURED". No access is checked at all.'

    SECURED_DEFAULT = u''

    PUBLIC_DEFAULT = u''

    def setUp(self):
        # Clean up
        try:
            settings.__delattr__('NOT_SECURED')
        except:
            pass
        try:
            settings.__delattr__('PUBLIC')
        except:
            pass
        try:
            settings.__delattr__('SECURED')
        except:
            pass

    def test_no_django_roles_tools_used(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        # expected = self.GREEN + '\n\t\t'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        # expected += self.ATTRIBUTES_OFF + '\n' + self.GREEN
        expected_2 += u'\n\t\tNo Django roles tool used. Access to '
        expected_2 += u'view depends on its implementation.'
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_no_django_roles_used_no_view_access_object_no_application(
            self
    ):
        expected_1 = u'\n\tAnalyzing {}:'.format(APP_NAME_FOR_NONE)
        expected_2 = u'Analysis for view: direct_view\n'
        expected_2 += u'\t\tView url: direct_view/'
        expected_2 += u'\n\t\tNo Django roles tool used. Access to '
        expected_2 += u'view depends on its implementation.'
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_no_django_roles_used_no_view_access_object_no_application_type(
            self
    ):
        expected_1 = u'\n\tAnalyzing {}:'.format(APP_NAME_FOR_NONE)
        expected_1 += u'\n\t\t{} has no type.'.format(APP_NAME_FOR_NONE)
        expected_2 = u'Analysis for view: direct_view\n'
        expected_2 += u'\t\tView url: direct_view/'
        expected_2 += u'\n\t\tNo Django roles tool used. Access to '
        expected_2 += u'view depends on its implementation.'
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_no_django_roles_used_no_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\tNo Django roles tool used. Access to '
        expected_2 += u'view depends on its implementation.'
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    def test_decorator_no_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\t' + NONE_TYPE_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_decorator_no_view_access_object_app_type_NOT_SECURED(self):
        settings.__setattr__('NOT_SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is NOT_SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\t' + NOT_SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('NOT_SECURED')

    def test_decorator_no_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\t' + SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    def test_decorator_no_view_access_object_app_type_PUBLIC(self):
        settings.__setattr__('PUBLIC', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is PUBLIC type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\t' + PUBLIC_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('PUBLIC')

    def test_decorator_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:view_protected_by_role',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_decorator_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:view_protected_by_role',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    def test_mixin_no_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:mixin_class_view\n'
        expected_2 += u'\t\tView url: role-included2/mixin_class_view/'
        expected_2 += u'\n\t\t' + NONE_TYPE_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_mixin_no_view_access_object_app_type_NOT_SECURED(self):
        settings.__setattr__('NOT_SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is NOT_SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:mixin_class_view\n'
        expected_2 += u'\t\tView url: role-included2/mixin_class_view/'
        expected_2 += u'\n\t\t' + NOT_SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('NOT_SECURED')

    def test_mixin_no_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:mixin_class_view\n'
        expected_2 += u'\t\tView url: role-included2/mixin_class_view/'
        expected_2 += u'\n\t\t' + SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    def test_mixin_no_view_access_object_app_type_PUBLIC(self):
        settings.__setattr__('PUBLIC', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is PUBLIC type.'
        expected_2 = u'Analysis for view: app-ns2:mixin_class_view\n'
        expected_2 += u'\t\tView url: role-included2/mixin_class_view/'
        expected_2 += u'\n\t\t' + PUBLIC_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('PUBLIC')

    def test_mixin_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:mixin_class_view\n'
        expected_2 += u'\t\tView url: role-included2/mixin_class_view/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:mixin_class_view',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    def test_mixin_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:view_protected_by_role\n'
        expected_2 += u'\t\tView url: role-included2/view_by_role/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:view_protected_by_role',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_no_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\t' + NONE_TYPE_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_no_view_access_object_app_type_NOT_SECURED(self):
        settings.__setattr__('NOT_SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is NOT_SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\t' + NOT_SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('NOT_SECURED')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_no_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\t' + SECURED_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_no_view_access_object_app_type_PUBLIC(self):
        settings.__setattr__('PUBLIC', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is PUBLIC type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\t' + PUBLIC_DEFAULT
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('PUBLIC')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_view_access_object_app_type_None(self):
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles has no type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:middleware_view_func',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_view_access_object_app_type_NOT_SECURED(self):
        settings.__setattr__('NOT_SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is NOT_SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\tView access is of type Public.'
        ViewAccess.objects.create(view='app-ns2:middleware_view_func',
                                  type='pu')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('NOT_SECURED')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_view_access_object_app_type_SECURED(self):
        settings.__setattr__('SECURED', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is SECURED type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\tView access is of type By role.'
        expected_2 += u'\n\t\t\tRoles with access: role-1, role-2'
        role_1, created = Group.objects.get_or_create(name='role-1')
        role_2, created = Group.objects.get_or_create(name='role-2')
        view_access = ViewAccess.objects.create(
            view='app-ns2:middleware_view_func',
            type='br')
        view_access.roles.add(role_1)
        view_access.roles.add(role_2)
        view_access.save()
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('SECURED')

    @modify_settings(MIDDLEWARE={
        'append': 'django_roles_access.middleware.RolesMiddleware'
    })
    def test_site_active_view_access_object_app_type_PUBLIC(self):
        settings.__setattr__('PUBLIC', ['django_roles'])
        expected_1 = u'\n\tAnalyzing django_roles:'
        expected_1 += u'\n\t\tdjango_roles is PUBLIC type.'
        expected_2 = u'Analysis for view: app-ns2:middleware_view_func\n'
        expected_2 += u'\t\tView url: role-included2/middleware_view_func/'
        expected_2 += u'\n\t\tView access is of type Authorized.'
        ViewAccess.objects.create(view='app-ns2:middleware_view_func',
                                  type='au')
        out = StringIO()
        call_command('checkviewaccess', stdout=out)
        self.assertIn(expected_1, out.getvalue())
        self.assertIn(expected_2, out.getvalue())
        settings.__delattr__('PUBLIC')




# encoding='utf-8'

from django.contrib import admin

from django_roles_access.models import ViewAccess, TemplateAccess


# admin.site.register(ViewAccess, ViewAccessAdmin)
admin.site.register(ViewAccess)
admin.site.register(TemplateAccess)



import unittest

from django.core.exceptions import PermissionDenied
from django.http import HttpResponseForbidden, HttpResponseRedirect

from django_roles_access.tools import DEFAULT_FORBIDDEN_MESSAGE
try:
    from unittest.mock import Mock, patch
except:
    from mock import Mock, patch
from django.conf import settings
from django.test import TestCase, RequestFactory, \
    modify_settings, override_settings
from django.contrib.auth import get_user_model

from django_roles_access.middleware import RolesMiddleware

User = get_user_model()


# UNIT TEST
@patch('django_roles_access.middleware.check_access_by_role')
class MiddlewareUnitTest(unittest.TestCase):

    def setUp(self):
        self.request = RequestFactory()
        self.request.path_info = Mock()
        self.request.user = Mock()
        self.middleware = RolesMiddleware(Mock())

    def test_init(
            self, mock_check_access_by_role=None
    ):
        init_result = RolesMiddleware('response')
        assert init_result.get_response == 'response'

    def test_middleware(
            self, mock_check_access_by_role
    ):
        response = self.middleware(self.request)
        assert response.django_roles

    def test_middleware_call_check_access_by_role(
            self, mock_check_access_by_role
    ):
        self.middleware(self.request)
        assert mock_check_access_by_role.called
        self.assertEqual(mock_check_access_by_role.call_count, 1)

    def test_middleware_call_check_access_by_roles_with_request(
            self, mock_check_access_by_role
    ):
        self.middleware(self.request)
        mock_check_access_by_role.assert_called_once_with(self.request)

    def test_middleware_get_response(
            self, mock_check_access_by_role
    ):
        self.middleware(self.request)
        self.middleware.get_response.assert_called_once_with(self.request)

    def test_middleware_return_response_with_request(
            self, mock_check_access_by_role
    ):
        def func(param):
            return param
        self.middleware.get_response.side_effect = func
        response = self.middleware(self.request)
        assert response == func(self.request)

    def test_middleware_return_http_forbidden_if_not_check_access(
            self, mock_check_access_by_role
    ):
        mock_check_access_by_role.return_value = False
        response = self.middleware(self.request)
        self.assertIsInstance(response, HttpResponseForbidden)

    def test_middleware_redirect_if_not_check_access(
            self, mock_check_access_by_role
    ):
        settings.__setattr__('DJANGO_ROLES_ACCESS_REDIRECT', True)
        mock_check_access_by_role.return_value = False
        response = self.middleware(self.request)
        settings.__delattr__('DJANGO_ROLES_ACCESS_REDIRECT')
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual(response.url, settings.LOGIN_URL)

    @patch('django_roles_access.middleware.get_no_access_response')
    def test_call_get_no_access_response_when_access_by_role_return_false(
            self, mock_get_no_access_response, mock_check_access_by_role
    ):
        mock_check_access_by_role.return_value = False
        self.middleware(self.request)
        assert mock_get_no_access_response.called

    @patch('django_roles_access.middleware.get_no_access_response')
    def test_call_once_get_no_access_response_when_access_by_role_return_false(
            self, mock_get_no_access_response, mock_check_access_by_role
    ):
        mock_check_access_by_role.return_value = False
        self.middleware(self.request)
        assert mock_get_no_access_response.call_count == 1


# INTEGRATED TEST
@modify_settings(MIDDLEWARE={
    'append': 'django_roles_access.middleware.RolesMiddleware'
})
class MiddlewareIntegratedTestSecuredApp(TestCase):

    def setUp(self):
        settings.__setattr__('SECURED', ['django_roles'])
        # User
        self.u1, created = User.objects.get_or_create(username='test-1')

    def tearDown(self):
        settings.__delattr__('SECURED')

    def test_get_access_view_function(self):
        self.client.force_login(self.u1)
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        self.assertEqual(response.status_code, 200)

    def test_get_access_denied_view_function(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        self.assertEqual(response.status_code, 403)

    def test_get_access_class_view(self):
        self.client.force_login(self.u1)
        response = self.client.get(
            '/role-included2/middleware_view_class/')
        self.assertEqual(response.status_code, 200)

    def test_get_access_denied_class_view(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_class/')
        self.assertEqual(response.status_code, 403)

    def test_default_behavior_when_no_configuration(self):
        settings.__delattr__('SECURED')
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        settings.__setattr__('SECURED', ['django_roles'])
        self.assertEqual(response.status_code, 200)

    def test_forbidden_behavior_without_configuration(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        expected = DEFAULT_FORBIDDEN_MESSAGE
        self.assertIn(expected, response.content.decode('utf-8'))

    def test_forbidden_behavior_with_configuration(self):
        expected = '<h1>No access</h1><p>Contact administrator</p>'
        settings.__setattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE', expected)
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        settings.__delattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE')
        self.assertIn(expected, response.content.decode('utf-8'))

    def test_redirect_if_configured(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_REDIRECT', True)
        self.client.logout()
        response = self.client.get(
            '/role-included2/middleware_view_func/')
        settings.__delattr__('DJANGO_ROLES_ACCESS_REDIRECT')
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, settings.LOGIN_URL)



from importlib import import_module
from unittest import TestCase as UnitTestCase

from django.contrib.auth.models import Group
from django.core.management import BaseCommand
from django.conf import settings
from django.test import TestCase
from django.views.generic import TemplateView
try:
    from unittest.mock import Mock, patch, MagicMock
except:
    from mock import Mock, patch

from django_roles_access.decorator import access_by_role
from django_roles_access.mixin import RolesMixin
from django_roles_access.models import ViewAccess
from tests import views
from django_roles_access.utils import (walk_site_url, get_views_by_app,
                                       view_access_analyzer, get_view_analyze_report,
                                       check_django_roles_is_used, print_view_analysis,
                                       analyze_by_role, APP_NAME_FOR_NONE,
                                       NOT_SECURED_DEFAULT, SECURED_DEFAULT,
                                       PUBLIC_DEFAULT, NONE_TYPE_DEFAULT)


class MockRegex:
    def __init__(self):
        self.pattern = '^fake-regex-pattern/$'


class MockRegexResolver:
    def __init__(self):
        self.pattern = '^fake-resolver/'


class MockRegexResolverNested:
    def __init__(self):
        self.pattern = '^fake-nested-resolver/'


class MockPattern:
    def __init__(self):
        self.regex = MockRegex()
        self.callback = 'fake-callback'
        self.name = 'fake-view-name'


class MockResolver:
    def __init__(self):
        self.url_patterns = [MockPattern()]
        self.regex = MockRegexResolver()
        self.app_name = 'fake-app-name'
        self.namespace = 'fake-namespace'


class MockResolverNested:
    def __init__(self):
        self.url_patterns = [MockResolver()]
        self.regex = MockRegexResolverNested()
        self.app_name = 'fake-app-name'
        self.namespace = 'nested-namespace'


class MockPatternDjango2:
    def __init__(self):
        self.pattern = '^fake-pattern/'
        self.callback = 'fake-callback'
        self.name = 'fake-view-name'


class MockPatternDjango2None:
    def __init__(self):
        self.pattern = '^fake-pattern/'
        self.callback = 'fake-callback'
        self.name = 'fake-view-none'


class MockResolverDjango2:
    def __init__(self):
        self.pattern = '^fake-resolver/'
        self.url_patterns = [MockPatternDjango2()]
        self.app_name = 'fake-app-name'
        self.namespace = 'fake-namespace'


class MockResolverDjango2None:
    def __init__(self):
        self.pattern = '^fake-resolver/'
        self.url_patterns = [MockPatternDjango2None()]
        self.app_name = None
        self.namespace = None


class MockResolverDjango2None2:
    def __init__(self):
        self.pattern = '^fake-resolver/'
        self.url_patterns = [MockResolverDjango2None()]
        self.app_name = 'fake-app-name'
        self.namespace = 'fake-namespace'


class MockResolverDjangoNested:
    def __init__(self):
        self.pattern = '^fake-nested-resolver/'
        self.url_patterns = [MockResolverDjango2()]
        self.app_name = 'fake-app-name'
        self.namespace = 'nested-namespace'


class UnitTestWalkSiteURL(UnitTestCase):

    def setUp(self):
        self.pattern_1 = MockPattern()
        self.data = [self.pattern_1]

    def test_second_param_is_optional_return_a_list(self):
        result = walk_site_url(self.data)
        self.assertIsInstance(result, list)

    def test_first_param_list_of_pattern_and_view(self):
        result = walk_site_url(self.data)
        self.assertEqual(result, [('fake-regex-pattern/', 'fake-callback',
                                   'fake-view-name', None)])

    def test_first_param_list_of_patterns_and_views(self):
        pattern_2 = MockPattern()
        pattern_2.regex.pattern = 'fake-regex-pattern-2/'
        pattern_2.callback = 'fake-view-2'
        result = walk_site_url([self.pattern_1, pattern_2])
        self.assertEqual(result, [('fake-regex-pattern/', 'fake-callback',
                                   'fake-view-name', None),
                                  ('fake-regex-pattern-2/', 'fake-view-2',
                                   'fake-view-name', None)])

    def test_param_list_with_pattern_and_resolver_django_1(self):
        expected_result = [
            ('fake-regex-pattern/', 'fake-callback', 'fake-view-name', None),
            ('fake-resolver/fake-regex-pattern/',
             'fake-callback', 'fake-namespace:fake-view-name', 'fake-app-name'
             )]
        resolver = MockResolver()
        result = walk_site_url([self.pattern_1, resolver])
        self.assertEqual(result, expected_result)

    def test_param_list_with_pattern_and_nested_resolver_django_1(self):
        expected_result = [
            ('fake-regex-pattern/', 'fake-callback', 'fake-view-name', None),
            ('fake-nested-resolver/fake-resolver/fake-regex-pattern/',
             'fake-callback', 'nested-namespace:fake-namespace:fake-view-name',
             'fake-app-name'
             )
        ]
        resolver = MockResolverNested()
        result = walk_site_url([self.pattern_1, resolver])
        self.assertEqual(result, expected_result)

    def test_param_list_with_pattern_and_resolver_django_2(self):
        expected_result = [
            ('fake-pattern/', 'fake-callback', 'fake-view-name', None),
            ('fake-resolver/fake-pattern/',
             'fake-callback', 'fake-namespace:fake-view-name', 'fake-app-name'
             )
        ]
        resolver = MockResolverDjango2()
        result = walk_site_url([MockPatternDjango2(), resolver])
        self.assertEqual(result, expected_result)

    def test_param_list_with_pattern_and_nested_resolver_django_2(self):
        expected_result = [
            ('fake-pattern/', 'fake-callback', 'fake-view-name', None),
            ('fake-nested-resolver/fake-resolver/fake-pattern/',
             'fake-callback', 'nested-namespace:fake-namespace:fake-view-name',
             'fake-app-name'
             )
        ]
        result = walk_site_url([MockPatternDjango2(),
                                MockResolverDjangoNested()])
        self.assertEqual(result, expected_result)

    def test_param_list_with_resolver_get_app_name_and_view_name_django_1(self):
        expected_result = [
            ('fake-resolver/fake-regex-pattern/',
             'fake-callback', 'fake-namespace:fake-view-name', 'fake-app-name'
             ),
            ('fake-nested-resolver/fake-resolver/fake-regex-pattern/',
             'fake-callback', 'nested-namespace:fake-namespace:fake-view-name',
             'fake-app-name'
             )
        ]
        result = walk_site_url([MockResolver(), MockResolverNested()])
        self.assertEqual(result, expected_result)

    def test_param_list_with_resolver_get_app_name_and_view_name_django_2(self):
        expected_result = [
            ('fake-resolver/fake-pattern/',
             'fake-callback', 'fake-namespace:fake-view-name', 'fake-app-name'
             ),
            ('fake-nested-resolver/fake-resolver/fake-pattern/',
             'fake-callback', 'nested-namespace:fake-namespace:fake-view-name',
             'fake-app-name'
             )
        ]
        resolver = MockResolverDjango2()
        nested_resolver = MockResolverDjangoNested()
        result = walk_site_url([resolver, nested_resolver])
        self.assertEqual(result, expected_result)

    def test_when_url_namespace_is_None(self):
        expected_result = [
            ('fake-resolver/fake-resolver/fake-pattern/',
             'fake-callback', 'fake-view-none', None
             )
        ]
        resolver = MockResolverDjango2None2()
        result = walk_site_url([resolver])
        self.assertEqual(result, expected_result)

    # def test_when_view_name_is_None(self):
    #     expected_result = [
    #         ('fake-resolver/fake-pattern/',
    #          'fake-callback', 'fake-view-name', None
    #          )
    #     ]
    #     resolver = MockResolverDjango2None2()
    #     result = walk_site_url([resolver])
    #     print(result)
    #     self.assertEqual(result, expected_result)


class IntegratedTestWalkSiteURL(TestCase):

    def setUp(self):
        self.url = import_module(settings.ROOT_URLCONF).urlpatterns

    def test_found_direct_access_view(self):
        expected_result = ('direct_access_view/',
                           views.protected_view_by_role,
                           'direct_access_view', None)
        result = walk_site_url(self.url)
        self.assertIn(expected_result, result)

    def test_found_included_view_without_namespace(self):
        expected_result = ('role-included[135]/view_by_role/',
                           views.protected_view_by_role,
                           'django_roles:view_protected_by_role',
                           'django_roles')
        result = walk_site_url(self.url)
        self.assertIn(expected_result, result)

    def test_found_included_view_with_namespace(self):
        expected_result = ('role-included2/view_by_role/',
                           views.protected_view_by_role,
                           'app-ns2:view_protected_by_role',
                           'django_roles')
        result = walk_site_url(self.url)
        self.assertIn(expected_result, result)

    def test_found_nested_access_view(self):
        expected_result = ('nest1/nest2/view_by_role/',
                           views.protected_view_by_role,
                           'nest1_namespace:nest2_namespace:view_'
                           'protected_by_role',
                           'roles-app-name')
        result = walk_site_url(self.url)
        self.assertIn(expected_result, result)


class UnitTestGetViewsByApp(UnitTestCase):
    """
    get_views_by_app receive the result of walk_site_url and is required to
    return a dictionary with keys been installed applications.
    """
    def setUp(self):
        self.data = [('a', 'b', 'c', 'fake-app-1')]

    @patch('django_roles_access.utils.settings')
    def test_returns_a_dictionary(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2']
        result = get_views_by_app(self.data)
        self.assertIsInstance(result, dict)

    @patch('django_roles_access.utils.settings')
    def test_returns_a_dictionary_with_all_installed_apps(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2']
        result = get_views_by_app(self.data)
        assert 'fake-app-1' in result
        assert 'fake-app-2' in result

    @patch('django_roles_access.utils.settings')
    def test_values_of_returned_dictionary_keys_are_lists(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2']
        result = get_views_by_app(self.data)
        self.assertIsInstance(result['fake-app-1'], list)
        self.assertIsInstance(result['fake-app-2'], list)

    @patch('django_roles_access.utils.settings')
    def test_receive_list_of_tuples_with_4_element(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1']
        result = get_views_by_app(self.data)
        assert 'fake-app-1' in result

    @patch('django_roles_access.utils.settings')
    def test_raise_type_error_if_receive_list_of_tuples_with_3_element(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1']
        data = [('a', 'b', 'c')]
        with self.assertRaises(TypeError):
            get_views_by_app(data)

    @patch('django_roles_access.utils.settings')
    def test_raise_type_error_if_receive_list_of_tuples_with_5_element(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1']
        data = [('a', 'b', 'c', 'd', 'e')]
        with self.assertRaises(TypeError):
            get_views_by_app(data)

    @patch('django_roles_access.utils.settings')
    def test_received_data_is_ordered_and_returned_by_application(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2', None]
        data = [('a', 'b', 'c', 'fake-app-1'), ('1', '2', '3', 'fake-app-2'),
                ('a1', 'b2', 'c3', None)]
        expected_result = [('a', 'b', 'c')]
        result = get_views_by_app(data)
        self.assertEqual(expected_result, result['fake-app-1'])

    @patch('django_roles_access.utils.settings')
    def test_can_work_with_no_declared_application_name(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2', None]
        data = [('a', 'b', 'c', 'fake-app-1'), ('1', '2', '3', 'fake-app-2'),
                ('a1', 'b2', 'c3', None)]
        expected_result = [('a1', 'b2', 'c3')]
        result = get_views_by_app(data)
        self.assertEqual(expected_result, result[APP_NAME_FOR_NONE])

    @patch('django_roles_access.utils.settings')
    def test_if_application_is_not_in_installed_apps_will_not_be_in_dict(
            self, mock_settings
    ):
        mock_settings.INSTALLED_APPS = ['fake-app-1', 'fake-app-2', None]
        result = get_views_by_app(self.data)
        assert 'fake-app-3' not in result


class IntegratedTestGetViewsByApp(TestCase):

    def setUp(self):
        self.url = import_module(settings.ROOT_URLCONF).urlpatterns
    
    def test_not_declared_app_are_recognized_as_undefined_app(self):
        expected_result = ('direct_access_view/',
                           views.protected_view_by_role,
                           'direct_access_view')
        result = get_views_by_app(walk_site_url(self.url))
        self.assertIn(expected_result, result[APP_NAME_FOR_NONE])

    def test_views_without_namespace_are_added_with_app_name_in_view_name(self):
        expected_result = ('role-included[135]/view_by_role/',
                           views.protected_view_by_role,
                           'django_roles:view_protected_by_role')
        result = get_views_by_app(walk_site_url(self.url))
        self.assertIn(expected_result, result['django_roles'])

    def test_view_with_namespace_are_added_with_correct_app_name(self):
        expected_result = ('role-included2/view_by_role/',
                           views.protected_view_by_role,
                           'app-ns2:view_protected_by_role')
        result = get_views_by_app(walk_site_url(self.url))
        self.assertIn(expected_result, result['django_roles'])

    def test_nested_namespace_are_added_with_correct_app_name(self):
        expected_result = ('nest1/nest2/view_by_role/',
                           views.protected_view_by_role,
                           'nest1_namespace:nest2_namespace:view_'
                           'protected_by_role')
        result = get_views_by_app(walk_site_url(self.url))
        self.assertIn(expected_result, result['roles-app-name'])


class TestViewAnalyzeReport(UnitTestCase):

    def test_report_for_no_application_type(self):
        expected = u'\t' + NONE_TYPE_DEFAULT
        result = get_view_analyze_report(None)
        self.assertEqual(result, expected)

    def test_report_for_application_type_NOT_SECURED(self):
        expected = u'\t' + NOT_SECURED_DEFAULT
        result = get_view_analyze_report('NOT_SECURED')
        self.assertEqual(result, expected)

    def test_report_for_application_type_SECURED(self):
        expected = u'\t' + SECURED_DEFAULT
        result = get_view_analyze_report('SECURED')
        self.assertEqual(result, expected)

    def test_report_for_application_type_PUBLIC(self):
        expected = u'\t' + PUBLIC_DEFAULT
        result = get_view_analyze_report('PUBLIC')
        self.assertEqual(result, expected)


class TestCheckDjangoRolesIsUsed(UnitTestCase):

    def test_detect_view_is_decorated(self):
        @access_by_role
        def function():
            pass
        self.assertTrue(check_django_roles_is_used(function))

    def test_detect_view_is_not_decorated(self):
        def function():
            pass
        self.assertFalse(check_django_roles_is_used(function()))

    def test_detect_view_use_mixin(self):
        class Aview(RolesMixin, TemplateView):
            template_name = 'dummyTemplate.html'
        self.assertTrue(check_django_roles_is_used(Aview))

    def test_detect_view_not_use_mixin(self):
        class Aview(TemplateView):
            template_name = 'dummyTemplate.html'
        self.assertFalse(check_django_roles_is_used(Aview))


@patch('django_roles_access.utils.ViewAccess')
class UnitTestAnalyzeByRoleAccess(UnitTestCase):

    def test_detect_access_is_by_role(
            self, mock_view_access
    ):
        expected = u'\n\t\t\tERROR: No roles configured to access de view.'
        mock_view_access.type = 'br'
        mock_view_access.roles.count.return_value = 0
        result = analyze_by_role(mock_view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_not_by_role(
            self, mock_view_access
    ):
        expected = u''
        mock_view_access.type = 'pu'
        result = analyze_by_role(mock_view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_not_by_role_with_roles(
            self, mock_view_access
    ):
        expected = u'\n\t\t\tRoles with access: role-1, role-2'
        mock_view_access.type = 'br'
        role_1 = Mock()
        role_1.name = u'role-1'
        role_2 = Mock()
        role_2.name = u'role-2'
        mock_view_access.roles.all.return_value = [role_1, role_2]
        result = analyze_by_role(mock_view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_not_by_role_without_roles(
            self, mock_view_access
    ):
        expected = u'\n\t\t\tERROR: No roles configured to access de view.'
        mock_view_access.type = 'br'
        mock_view_access.roles.count.return_value = 0
        result = analyze_by_role(mock_view_access)
        self.assertEqual(result, expected)


class IntegratedTestAnalyzeByRoleAccess(TestCase):

    def test_detect_access_is_by_role(self):
        expected = u'\n\t\t\tERROR: No roles configured to access de view.'
        view_access = ViewAccess.objects.create(view='any-name', type='br')
        result = analyze_by_role(view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_not_by_role(self):
        expected = u''
        view_access = ViewAccess.objects.create(view='any-name', type='pu')
        result = analyze_by_role(view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_by_role_with_roles(self):
        expected = u'\n\t\t\tRoles with access: role-1, role-2'
        view_access = ViewAccess.objects.create(view='any-name', type='br')
        role_1, created = Group.objects.get_or_create(name='role-1')
        role_2, created = Group.objects.get_or_create(name='role-2')
        view_access.roles.add(role_1)
        view_access.roles.add(role_2)
        view_access.save()
        result = analyze_by_role(view_access)
        self.assertEqual(result, expected)

    def test_detect_access_is_not_by_role_without_roles(self):
        expected = u'\n\t\t\tERROR: No roles configured to access de view.'
        view_access = ViewAccess.objects.create(view='any-name', type='br')
        result = analyze_by_role(view_access)
        self.assertEqual(result, expected)


@patch('django_roles_access.utils.ViewAccess.objects')
class UnitTestViewAnalyzer(UnitTestCase):

    def test_view_analyzer_return_a_report(
            self, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        result = view_access_analyzer('fake-app-type', 'fake-callback',
                                      'fake-view-name', 'fake-site-active')
        try:
            self.assertIsInstance(result, unicode)
        except:
            self.assertIsInstance(result, str)

    def test_view_analyzer_search_view_access_for_the_view(
            self, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', 'fake-site-active')
        assert mock_objects.first.called

    def test_view_analyzer_search_view_access_for_the_view_once(
            self, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', 'fake-site-active')
        self.assertEqual(mock_objects.filter.call_count, 1)

    def test_view_analyzer_search_view_access_with_view_name(
            self, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', 'fake-site-active')
        mock_objects.filter.assert_called_once_with(view='fake-view-name')

    def test_view_access_type_when_site_active_and_exists_view_access(
            self, mock_objects
    ):
        expected = u'\tView access is of type Public.'
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        result = view_access_analyzer('fake-app-type', 'fake-callback',
                                      'fake-view-name', True)
        self.assertEqual(result, expected)

    @patch('django_roles_access.utils.analyze_by_role')
    def test_view_access_type_by_role_call_analyze_by_role(
            self, mock_analyze_by_role, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'br'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', True)
        assert mock_analyze_by_role.called

    @patch('django_roles_access.utils.analyze_by_role')
    def test_view_access_type_by_role_call_analyze_by_role_once(
            self, mock_analyze_by_role, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'br'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', True)
        self.assertEqual(mock_analyze_by_role.call_count ,1)

    @patch('django_roles_access.utils.analyze_by_role')
    def test_view_access_type_by_role_call_analyze_by_role_with_view_access(
            self, mock_analyze_by_role, mock_objects
    ):
        view_access = Mock()
        view_access.type = 'br'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        view_access_analyzer('fake-app-type', 'fake-callback',
                             'fake-view-name', True)
        mock_analyze_by_role.assert_called_once_with(view_access)

    def test_no_view_access_object_for_the_view_and_site_active_no_app_type(
            self, mock_objects
    ):
        expected = u'\t' + NONE_TYPE_DEFAULT
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer(None, 'fake-callback',
                                      'fake-view-name', True)
        self.assertEqual(result, expected)

    def test_no_view_access_object_and_site_active_app_type_NOT_SECURED(
            self, mock_objects
    ):
        expected = u'\t' + NOT_SECURED_DEFAULT
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer('NOT_SECURED', 'fake-callback',
                                      'fake-view-name', True)
        self.assertEqual(result, expected)

    def test_no_view_access_object_and_site_active_app_type_SECURED(
            self, mock_objects
    ):
        expected = u'\t' + SECURED_DEFAULT
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer('SECURED', 'fake-callback',
                                      'fake-view-name', True)
        self.assertEqual(result, expected)

    def test_no_view_access_object_and_site_active_app_type_PUBLIC(
            self, mock_objects
    ):
        expected = u'\t' + PUBLIC_DEFAULT
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer('PUBLIC', 'fake-callback',
                                      'fake-view-name', True)
        self.assertEqual(result, expected)

    def test_middleware_not_used_view_access_object_exist_and_dr_tools_used(
            self, mock_objects
    ):
        expected = u'\tView access is of type Public.'

        @access_by_role
        def function():
            pass
        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        result = view_access_analyzer('fake-app-type', function,
                                      'fake-view-name', False)
        self.assertEqual(result, expected)

    def test_middleware_not_used_view_access_object_exist_and_dr_tools_not_used(
            self, mock_objects
    ):
        expected = u'\tERROR: View access object exist for the view, but no '
        expected += u'Django role tool is used: neither decorator, mixin, or '
        expected += u'middleware.'

        def function():
            pass

        view_access = Mock()
        view_access.type = 'pu'
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = view_access
        result = view_access_analyzer('fake-app-type', function,
                                      'fake-view-name', False)
        self.assertEqual(result, expected)

    def test_middleware_not_used_dr_tools_are_used_no_view_access_object(
            self, mock_objects
    ):
        expected = u'\t' + PUBLIC_DEFAULT

        @access_by_role
        def function():
            pass
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer('PUBLIC', function,
                                      'fake-view-name', False)
        self.assertEqual(result, expected)

    def test_no_django_roles_tools_used_no_application_type(
            self, mock_objects
    ):
        expected = u'\tNo Django roles tool used. Access to view depends on '
        expected += u'its implementation.'

        def function():
            pass
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer(None, function,
                                      'fake-view-name', False)
        self.assertEqual(result, expected)

    def test_no_django_roles_tools_used_application_type(
            self, mock_objects
    ):
        expected = u'\tNo Django roles tool used. Access to view depends on '
        expected += u'its implementation.'

        def function():
            pass
        mock_objects.filter.return_value = mock_objects
        mock_objects.first.return_value = None
        result = view_access_analyzer('Authorized', function,
                                      'fake-view-name', False)
        self.assertEqual(result, expected)


class IntegratedTestViewAnalyzezr(TestCase):

    def test_with_middleware_SECURED_without_view_access_object(self):
        expected = u'\t' + SECURED_DEFAULT
        result = view_access_analyzer('SECURED', views.MiddlewareView.as_view,
                                      'django_roles:middleware_view_class',
                                      True)
        self.assertEqual(expected, result)

    def test_with_middleware_with_view_access_object(self):
        expected = u'\tView access is of type By role.'
        expected += u'\n\t\t\tERROR: No roles configured to access de view.'
        ViewAccess.objects.create(view='django_roles:middleware_view_class',
                                  type='br')
        result = view_access_analyzer('SECURED', views.MiddlewareView.as_view,
                                      'django_roles:middleware_view_class',
                                      True)
        self.assertEqual(result, expected)

    def test_with_middleware_with_view_access_object_with_roles(self):
        expected = u'\tView access is of type By role.'
        expected += u'\n\t\t\tRoles with access: test1, test2'
        g1, created = Group.objects.get_or_create(name='test1')
        g2, created = Group.objects.get_or_create(name='test2')
        view_access = ViewAccess.objects.create(
            view='django_roles:middleware_view_class',
            type='br')
        view_access.roles.add(g1)
        view_access.roles.add(g2)
        view_access.save()
        result = view_access_analyzer('SECURED', views.MiddlewareView.as_view,
                                      'django_roles:middleware_view_class',
                                      True)
        self.assertEqual(result, expected)

    def test_with_middleware_with_view_access_object_authorized(self):
        expected = u'\tView access is of type Authorized.'
        ViewAccess.objects.create(view='django_roles:middleware_view_class',
                                  type='au')
        result = view_access_analyzer('SECURED', views.MiddlewareView.as_view,
                                      'django_roles:middleware_view_class',
                                      True)
        self.assertEqual(result, expected)

    def test_with_middleware_with_view_access_object_public(self):
        expected = u'\tView access is of type Public.'
        ViewAccess.objects.create(view='django_roles:middleware_view_class',
                                  type='pu')
        result = view_access_analyzer('SECURED', views.MiddlewareView.as_view,
                                      'django_roles:middleware_view_class',
                                      True)
        self.assertEqual(result, expected)

    def test_without_middleware_with_view_access_object(self):
        expected = u'\tView access is of type By role.'
        expected += u'\n\t\t\tERROR: No roles configured to access de view.'
        ViewAccess.objects.create(view='django_roles:view_protected_by_role',
                                  type='br')
        result = view_access_analyzer('SECURED', views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_with_view_access_object_with_roles(self):
        expected = u'\tView access is of type By role.'
        expected += u'\n\t\t\tRoles with access: test1, test2'
        g1, created = Group.objects.get_or_create(name='test1')
        g2, created = Group.objects.get_or_create(name='test2')
        view_access = ViewAccess.objects.create(
            view='django_roles:view_protected_by_role',
            type='br')
        view_access.roles.add(g1)
        view_access.roles.add(g2)
        view_access.save()
        result = view_access_analyzer('SECURED', views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_with_view_access_object_authorized(self):
        expected = u'\tView access is of type Authorized.'
        ViewAccess.objects.create(view='django_roles:view_protected_by_role',
                                  type='au')
        result = view_access_analyzer('SECURED', views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_with_view_access_object_public(self):
        expected = u'\tView access is of type Public.'
        ViewAccess.objects.create(view='django_roles:view_protected_by_role',
                                  type='pu')
        result = view_access_analyzer('SECURED', views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_without_view_access_object_and_view_protected(
            self
    ):
        expected = u'\t' + SECURED_DEFAULT
        result = view_access_analyzer('SECURED', views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_no_view_access_object_and_view_protected_without_app(
            self
    ):
        expected = u'\t' + NONE_TYPE_DEFAULT

        result = view_access_analyzer(None, views.protected_view_by_role,
                                      'django_roles:view_protected_by_role',
                                      False)
        self.assertEqual(result, expected)

    def test_without_middleware_with_view_access_object_and_view_not_protected(
            self
    ):
        expected = u'\tERROR: View access object exist for the view, '
        expected += 'but no Django role tool is used: neither '
        expected += 'decorator, mixin, or middleware.'
        ViewAccess.objects.create(view='django_roles:middleware_view_func',
                                  type='pu')
        result = view_access_analyzer(None, views.middleware_view,
                                      'django_roles:middleware_view_func',
                                      False)
        self.assertEqual(result, expected)


@patch.object(BaseCommand(), 'style')
@patch.object(BaseCommand(), 'stdout')
class UnitTestPrintViewAnalysis(UnitTestCase):

    def test_call_stdout(
            self, mock_stdout, mock_style
    ):
        # output = StringIO()
        # sys.stdout = output
        print_view_analysis(mock_stdout, mock_style, 'fake report')
        assert mock_stdout.write.called

    def test_call_stdout_once(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'fake report')
        self.assertEqual(mock_stdout.write.call_count, 1)

    def test_call_stdout_with_SUCCESS(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'fake report')
        mock_stdout.write.assert_called_once_with(
            mock_style.SUCCESS())

    def test_call_SUCCESS_style(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'fake report')
        assert mock_style.SUCCESS.called

    def test_call_SUCCESS_style_with_report(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'fake report')
        mock_style.SUCCESS.assert_called_once_with('\t' + 'fake report')

    def test_call_ERROR_style_when_there_is_an_error_in_report(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'ERROR: fake report')
        assert mock_style.ERROR.called

    def test_call_ERROR_style_when_there_is_an_error_in_report_with_report(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'ERROR: fake report')
        mock_style.ERROR.assert_called_once_with('\t' + 'ERROR: fake report')

    def test_call_WARNING_style_when_there_is_a_warning_in_report(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'WARNING: fake report')
        assert mock_style.WARNING.called

    def test_call_WARNING_style_when_there_is_a_warning_in_report_with_report(
            self, mock_stdout, mock_style
    ):
        print_view_analysis(mock_stdout, mock_style, 'WARNING: fake report')
        mock_style.WARNING.assert_called_once_with(
            '\t' + 'WARNING: fake report')



from django_roles_access.tools import check_access_by_role, get_no_access_response


class RolesMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if not check_access_by_role(request):
            return get_no_access_response()

        response = self.get_response(request)

        # Only useful for unit test.
        response.django_roles = True

        return response



from unittest import TestCase as UnitTestCase
from django.template import Template, Context
from django.test import TestCase, RequestFactory
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group


try:
    from unittest.mock import Mock, patch
except:
    from mock import Mock, patch

from django_roles_access.models import TemplateAccess
from django_roles_access.templatetags.roles_tags import check_role

User = get_user_model()


class UnitTestRolesTags(UnitTestCase):

    def test_roles_tags_show_content_for_superuser(self):
        user = Mock()
        user.is_superuser = True
        self.assertTrue(check_role(user=user, flag='fake-flag'))

    @patch('django_roles_access.templatetags.roles_tags.TemplateAccess.objects')
    def test_roles_tags_show_content_for_flag_and_user_in_roles(
            self, mock_ta_objects
    ):
        user = Mock()
        user.is_superuser = False
        user.groups.all.return_value = ['fake-group']
        template_flag = Mock()
        template_flag.roles.all.return_value = ['fake-group', 'other-group']
        mock_ta_objects.get.return_value = template_flag
        self.assertTrue(check_role(user=user, flag='fake-flag'))

    @patch('django_roles_access.templatetags.roles_tags.TemplateAccess.objects')
    def test_roles_tags_not_show_content_for_flag_and_user_not_in_roles(
            self, mock_ta_objects
    ):
        user = Mock()
        user.is_superuser = False
        user.groups.all.return_value = ['no-group']
        template_flag = Mock()
        template_flag.roles.all.return_value = ['fake-group', 'other-group']
        mock_ta_objects.get.return_value = template_flag
        self.assertFalse(check_role(user=user, flag='fake-flag'))


class IntegratedTestRolesTags(TestCase):

    TEMPLATE = Template("{% load roles_tags %} "
                        "{% if request.user|check_role:'test-flag' %} "
                        "checked access {% endif %} ")

    def setUp(self):
        self.user, created = User.objects.get_or_create(username='test')
        self.group, created = Group.objects.get_or_create(name='Test-group')
        self.request = RequestFactory()
        self.request.user = self.user

    def test_template_tag_show_content(self):
        template_acces, created = TemplateAccess.objects.get_or_create(
            flag='test-flag')
        template_acces.roles.add(self.group)
        template_acces.save()
        self.user.groups.add(self.group)
        self.user.save()
        rendered = self.TEMPLATE.render(Context({'request': self.request}))
        self.assertIn('checked access', rendered)

    def test_template_tag_show_content_if_superuser(self):
        template_acces, created = TemplateAccess.objects.get_or_create(
            flag='test-flag')
        template_acces.roles.add(self.group)
        template_acces.save()
        self.user.is_superuser = True
        rendered = self.TEMPLATE.render(Context({'request': self.request}))
        self.assertIn('checked access', rendered)

    def test_template_tag_do_not_show_content_if_no_template_access(self):
        rendered = self.TEMPLATE.render(Context({}))
        self.assertNotIn('checked access', rendered)

    def test_template_tag_do_not_show_content_if_user_not_in_roles(self):
        template_acces, created = TemplateAccess.objects.get_or_create(
            flag='test-flag')
        template_acces.roles.add(self.group)
        template_acces.save()
        self.user.groups.remove(self.group)
        self.user.save()
        rendered = self.TEMPLATE.render(Context({'request': self.request}))
        self.assertNotIn('checked access', rendered)



# encoding='utf-8'
# Translation https://stackoverflow.com/questions/2938692/
# django-internationalization-for-admin-pages-translate-model-name-and-attribute

from django.db import models
try:
    from django.utils.translation import gettext_lazy as _
except:
    from django.utils.translation import ugettex_lazy as _
from django.contrib.auth.models import Group


class ViewAccess(models.Model):
    """
    Implements security in views having a name.

    The view name is *view* attribute value. The security for view name is
    configured with next two attributes: type (type of security) and roles
    (only if type is By role).
    """

    PUBLIC = 'pu'
    AUTHORIZED = 'au'
    BY_ROLE = 'br'

    ACCESS_TYPES = (
        (PUBLIC, _('Public')),
        (AUTHORIZED, _('Authorized')),
        (BY_ROLE, _('By role'))
    )

    #: View's name to be secured. Value can be just a view name 'objects_a',
    #: with application name: 'app_name:index', or a nested configurations with
    #: namespaces: 'namespace_1:namespace_2:index'.
    view = models.CharField(max_length=255, unique=True, default=None,
                            help_text=_(u'View name to be secured: '
                                        u'<em>namespace:view_name</em>'),
                            verbose_name=_(u'View'))
    #: The type of access as defined by ACCESS_TYPES
    type = models.CharField(max_length=2, choices=ACCESS_TYPES, default=None,
                            help_text=_(u'Type of access for the view. '
                                        u'Select from available options.'),
                            verbose_name=_(u'Type'))
    #: One or more roles (:func:`django.contrib.auth.models.Group`) with access.
    #: This attributes can always be empty. If the type selected is *By role*
    #: and no roles are added to this attribute, then *checkviewaccess* will
    #: report an ERROR.
    roles = models.ManyToManyField(Group, help_text=
                                   _(u'Select the groups (roles) with view '
                                     u'access if access type = By role.'),
                                   related_name='view_access',
                                   verbose_name=_(u'Roles'),
                                   blank=True)

    class Meta:
        verbose_name = _('View access')
        verbose_name_plural = _('Views access')

    def __str__(self):
        return self.view


class TemplateAccess(models.Model):
    """
    Implements content security.

    This model let administrator to add or remove Groups (roles) from *roles*
    attribute. In templates, users belonging to any added group will pass the
    check.
    """
    #: Template's flag, text label, to identify each template access object.
    #: Flag must be unique.
    flag = models.CharField(max_length=255, unique=True, default=None,
                            help_text=_(u'Unique between all applications.'
                                        u'Flag is used with template tag '
                                        u'check_role to restrict access in '
                                        u'templates.'),
                            verbose_name=_(u'Flag'))
    #: One or more roles (:func:`django.contrib.auth.models.Group`) with access.
    roles = models.ManyToManyField(Group, help_text=
                                   _(u'Select the groups (roles) with access '
                                     u'with check_role template tag and '
                                     u'flag.'),
                                   related_name='template_access',
                                   verbose_name=_(u'Roles'))

    class Meta:
        verbose_name = _('Template access')
        verbose_name_plural = _('Templates access')

    def __str__(self):
        return self.flag



from django.apps import apps
from django.test import TestCase
from django_roles_access.apps import RolesConfig


class ReportsConfigTest(TestCase):
    def test_apps(self):
        # Test the class
        self.assertEqual(RolesConfig.name, 'django_roles_access')
        self.assertEqual(RolesConfig.verbose_name, u'Django Roles Access')

        # Test the app
        self.assertEqual(apps.get_app_config('django_roles_access').name,
                         'django_roles_access')



"""
Code used by checkviewaccess management command
"""
from django.conf import settings
from django.contrib.auth import get_user_model
try:
    from django.utils.translation import gettext as _
except:
    from django.utils.translation import ugettext as _

from django_roles_access.models import ViewAccess

User = get_user_model()
APP_NAME_FOR_NONE = _(u'Undefined app')

NOT_SECURED_DEFAULT = _(u'WARNING: View has no security configured '
                        u'(ViewAccess) and application type is "NOT_SECURED".'
                        u' No access is checked at all.')

SECURED_DEFAULT = _(u'No security configured for the view (ViewAccess '
                    u'object) and application type is "SECURED". User is '
                    u'required to be authenticated to access the view.')

PUBLIC_DEFAULT = _(u'No security configured for the view (ViewAccess object)'
                   u' and application type is "PUBLIC". Anonymous user can'
                   u' access the view.')

NONE_TYPE_DEFAULT = _(u'ERROR: Django roles middleware is active; or view'
                      u' is protected with Django roles decorator or mixin,'
                      u' and has no application or application has no type. '
                      u'There are no View Access object for the view. Is not '
                      u'possible to determine behavior for access view. Access'
                      u' to view is determined by view implementation.')


def walk_site_url(_url_patterns, recursive_url='',
                  view_name=None, app_name=None):
    result = []
    for url in _url_patterns:
        if hasattr(url, 'pattern'):
            # Running With Django 2
            pattern = str(url.pattern)
        else:
            # Running with Django 1
            pattern = str(url.regex.pattern)
        pattern = pattern.strip('^').strip('$')  # For better presentation
        if hasattr(url, 'url_patterns'):
            # When url object has 'url_patterns' attribute means is a Resolver
            if url.namespace:
                if view_name:
                    new_view_name = view_name + ":" + url.namespace
                else:
                    new_view_name = url.namespace
            else:
                new_view_name = None
            result.extend(walk_site_url(url.url_patterns,
                                        recursive_url + pattern,
                                        new_view_name, url.app_name))
        else:
            if view_name:
                new_view_name = view_name + ":" + url.name
            else:
                new_view_name = url.name
            result.append((recursive_url + pattern, url.callback,
                           new_view_name, app_name))

    return result


def get_views_by_app(site_urls):
    installed_apps = settings.INSTALLED_APPS
    result = {key: [] for key in installed_apps}
    for site_url in site_urls:
        try:
            url, callback, view_name, app_name = site_url
        except:
            raise TypeError
        if not app_name:
            app_name = APP_NAME_FOR_NONE
        try:
            result[app_name].append((url, callback, view_name))
        except KeyError:
            result[app_name] = [(url, callback, view_name)]
    return result


def get_view_analyze_report(app_type):
    if app_type == 'NOT_SECURED':
        return u'\t' + NOT_SECURED_DEFAULT
    elif app_type == 'SECURED':
        return u'\t' + SECURED_DEFAULT
    elif app_type == 'PUBLIC':
        return u'\t' + PUBLIC_DEFAULT
    else:
        return u'\t' + NONE_TYPE_DEFAULT


def check_django_roles_is_used(view):
    if hasattr(view, 'access_by_role'):
        return True
    elif 'dispatch' in dir(view):
        if hasattr(view.dispatch, 'access_by_role'):
            return True
    return False


def analyze_by_role(view_access):
    result = u''
    if view_access.type == 'br':
        if view_access.roles.count() != 0:
            result = _(u'\n\t\t\tRoles with access: ')
            for role in view_access.roles.all():
                result += role.name + u', '
            result = result[:-2]
        else:
            result = _(u'\n\t\t\tERROR: No roles configured to access de view.')
    return result


def view_access_analyzer(app_type, callback, view_name, site_active):
    result = _(u'\tNo Django roles tool used. Access to view depends on '
               u'its implementation.')
    view_access = ViewAccess.objects.filter(view=view_name).first()
    if site_active:
        if view_access:
            view_access_type = dict(ViewAccess.ACCESS_TYPES)[view_access.type]
            result = _(u'\tView access is of type {}.'.format(view_access_type))
            result += analyze_by_role(view_access)
        else:
            result = get_view_analyze_report(app_type)
    else:
        if check_django_roles_is_used(callback):
            if view_access:
                view_access_type = \
                    dict(ViewAccess.ACCESS_TYPES)[view_access.type]
                result = _(
                    u'\tView access is of type {}.'.format(view_access_type))
                result += analyze_by_role(view_access)
            else:
                result = get_view_analyze_report(app_type)
        else:
            if view_access:
                result = _(u'\tERROR: View access object exist for the view, '
                           u'but no Django role tool is used: neither '
                           u'decorator, mixin, or middleware.')
    return result


def print_view_analysis(stdout, style, report):
    if 'ERROR' in report:
        stdout.write(style.ERROR('\t' + report))
    elif 'WARNING' in report:
        stdout.write(style.WARNING('\t' + report))
    else:
        stdout.write(style.SUCCESS('\t' + report))



from django import template
from django_roles_access.models import TemplateAccess

register = template.Library()


@register.filter(name='check_role')
def check_role(user, flag):
    """
    **flag** is a unique string used to restrict access by role in template
    content. With **flag** is recover an :class:`roles.models.TemplateAccess`
    object.

    :param flag: :attribute:`roles.models.TemplateAccess.flag`.
    :param user:
    :return:
    """
    try:
        if user.is_superuser:
            return True
        template_flag = TemplateAccess.objects.get(flag__exact=flag)
        for group in user.groups.all():
            if group in template_flag.roles.all():
                return True
        return False
    except:
        return False



# -*- coding: utf-8 -*-
"""
This test module will search for all site's urls and analyze their security
status.
"""
from importlib import import_module
from django.core.management.base import BaseCommand
try:
    from django.utils.translation import gettext as _
except:
    from django.utils.translation import ugettext as _
from django.conf import settings

from django_roles_access.tools import get_app_type
from django_roles_access.utils import (walk_site_url, get_views_by_app,
                                       view_access_analyzer, print_view_analysis)


DJANGO_ROLE_MIDDLEWARE = 'django_roles_access.middleware.RolesMiddleware'


class Command(BaseCommand):
    """
    **checkviewaccess** management command analyze *all site's views access*.

    The list with *all site's views* is obtained from *urlpatterns* attribute of
    the URLConf module configured in settings.ROOT_URLCONF.Each view access from
    the list of *site's views* is subject of analyze.

    Conclusion of analyze is reported by application as Django Roles let
    classify installed applications.
    """
    help = 'Manage command checkviewaccess'

    # def add_arguments(self, parser):
    #     """
    #     The command can receive arguments. They have to be
    #     declared in add_arguments method.
    #     """
    #     parser.add_argument('ids', nargs='+', type=int)

    def handle(self, *args, **options):
        """
        This method implements checkviewaccess command behavior.
        """
        self.stdout.write(self.style.SUCCESS(
            _(u'Start checking views access.')))

        # 1. All views are collected and grouped by application
        self.stdout.write(self.style.SUCCESS(
            _(u'Start gathering information.')))
        url = import_module(settings.ROOT_URLCONF).urlpatterns
        views_by_app = get_views_by_app(walk_site_url(url))
        self.stdout.write(self.style.SUCCESS(
            _(u'Finish gathering information.')))

        # 2. Check if Django roles middleware is active or not
        if DJANGO_ROLE_MIDDLEWARE in settings.MIDDLEWARE:
            site_active = True
        else:
            site_active = False
        self.stdout.write(
            self.style.SUCCESS(
                _(u'Django roles active for site: {}.').format(site_active)))

        # 3. Analysis is done by application
        for app_name, views_list in views_by_app.items():
            self.stdout.write(
                self.style.SUCCESS(
                    _(u'\tAnalyzing {}:').format(app_name)))
            # Get application classification.
            app_type = get_app_type(app_name)
            if app_type:
                self.stdout.write(
                    self.style.SUCCESS(
                        _(u'\t\t{} is {} type.').format(app_name, app_type)))
            else:
                self.stdout.write(
                    self.style.WARNING(
                        _(u'\t\t{} has no type.').format(app_name)))

            # if application does not have views list:
            if len(views_list) == 0:
                self.stdout.write(
                    self.style.WARNING(
                        _(u'\t\t{} does not have configured views.'.format(
                            app_name))))

            # 4. For each view of the analyzed application
            for url, callback, view_name in views_list:
                self.stdout.write(
                    self.style.SUCCESS(
                        _(u'\n\t\tAnalysis for view: {}'
                          u'\n\t\tView url: {}'.format(view_name, url))))

                analysis = view_access_analyzer(app_type, callback, view_name,
                                                site_active)

                print_view_analysis(self.stdout, self.style, analysis)

            # 5. End for app_name in views_by_app:
            self.stdout.write(
                self.style.SUCCESS(
                    _(u'\tFinish analyzing {}.').format(app_name)))

        # 6. End of report
        self.stdout.write(self.style.SUCCESS(
            _(u'End checking view access.')))



from django.utils.decorators import method_decorator

from django_roles_access.decorator import access_by_role


class RolesMixin(object):
    """
    A mixin that user access_by_role decorator for dispatch method.
    """

    @method_decorator(access_by_role)
    def dispatch(self, request, *args, **kwargs):
        return super(RolesMixin, self).dispatch(request, *args, **kwargs)



import unittest
from django.conf import settings
from django.test import TestCase, override_settings
from django.contrib.auth import get_user_model

try:
    from unittest.mock import Mock, patch
except:
    from mock import Mock, patch

from tests.views import ProtectedMixinView

User = get_user_model()


# UNIT TEST
class TestUnitRolesMixin(unittest.TestCase):

    def test_preserve_attributes(
            self
    ):
        dispatched = ProtectedMixinView().dispatch
        self.assertIs(getattr(dispatched, 'access_by_role',
                      False), True)


# INTEGRATED TEST
class TestIntegratedRolesMixin(TestCase):

    def setUp(self):
        settings.__setattr__('SECURED', ['django_roles'])
        # User
        self.u1, created = User.objects.get_or_create(username='test-1')

    def tearDown(self):
        settings.__delattr__('SECURED')

    def test_get_access_class_view(self):
        self.client.force_login(self.u1)
        response = self.client.get(
            '/role-included2/mixin_class_view/')
        self.assertEqual(response.status_code, 200)

    def test_get_access_denied_class_view(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/mixin_class_view/')
        self.assertEqual(response.status_code, 403)



# Generated by Django 2.0.7 on 2019-03-19 15:07

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='TemplateAccess',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('flag', models.CharField(default=None, help_text='Unique between all applications.Flag is used with template tag check_role to restrict access in templates.', max_length=255, unique=True, verbose_name='Flag')),
                ('roles', models.ManyToManyField(help_text='Select the groups (roles) with access with check_role template tag and flag.', related_name='template_access', to='auth.Group', verbose_name='Roles')),
            ],
            options={
                'verbose_name': 'Template access',
                'verbose_name_plural': 'Templates access',
            },
        ),
        migrations.CreateModel(
            name='ViewAccess',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('view', models.CharField(default=None, help_text='View name to be secured: <em>namespace:view_name</em>', max_length=255, unique=True, verbose_name='View')),
                ('type', models.CharField(choices=[('pu', 'Public'), ('au', 'Authorized'), ('br', 'By role')], default=None, help_text='Type of access for the view. Select from available options.', max_length=2, verbose_name='Type')),
                ('roles', models.ManyToManyField(blank=True, help_text='Select the groups (roles) with view access if access type = By role.', related_name='view_access', to='auth.Group', verbose_name='Roles')),
            ],
            options={
                'verbose_name': 'View access',
                'verbose_name_plural': 'Views access',
            },
        ),
    ]



from functools import wraps

from django_roles_access.tools import check_access_by_role, get_no_access_response


def access_by_role(view):
    """
    Check if logged user can access the decorated function or method.
    """
    @wraps(view)
    def _view(request, *args, **kwargs):
        if check_access_by_role(request):
            return view(request, *args, **kwargs)
        return get_no_access_response()

    _view.access_by_role = True
    return _view




SECRET_KEY = 'django-roles'

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django_roles_access',
]

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

import os
BASE_DIR = os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))
LOCALE_PATHS = [
    os.path.join(BASE_DIR, "django_roles_access/locale"),
]



import unittest
from django.conf import settings
from django.test import TestCase
from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import (user_passes_test,
                                            login_required,
                                            permission_required)
try:
    from unittest.mock import Mock, patch, MagicMock
except:
    from mock import Mock, patch

from django_roles_access.models import ViewAccess
from django_roles_access.decorator import access_by_role
from django_roles_access.tools import DEFAULT_FORBIDDEN_MESSAGE

User = get_user_model()


def myattr_dec(func):
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    wrapper.myattr = True
    return wrapper


class TestAccessByRoleWithOtherDecorators(unittest.TestCase):

    def test_view_is_decorated_with_access_by_role(self):
        from .views import protected_view_by_role
        self.assertTrue(protected_view_by_role.access_by_role)

    def test_attributes(self):
        doc = "Check if logged user can access the decorated function or " \
              "method."
        self.assertEqual(access_by_role.__name__, 'access_by_role')
        self.assertIn(doc, access_by_role.__doc__)
        self.assertEqual(access_by_role.__dict__, {})

    def test_simple_preserve_attributes_with_other_decorator(self):
        @access_by_role
        def func():
            pass
        self.assertIs(getattr(func, 'access_by_role', False), True)

        @access_by_role
        @myattr_dec
        def func():
            pass

        self.assertIs(getattr(func, 'myattr', False), True)
        self.assertIs(getattr(func, 'access_by_role', False), True)

    def test_preserve_attributes_with_contrib_user_passes_test(self):

        def challenge(user):
            return True

        @access_by_role
        @user_passes_test(challenge)
        def func():
            pass
        self.assertIs(getattr(func, 'access_by_role', False), True)

    def test_preserve_attributes_with_contrib_login_required(self):
        @login_required
        @access_by_role
        def func():
            pass
        self.assertIs(getattr(func, 'access_by_role', False), True)

    def test_preserve_attributes_with_contrib_permission_required(self):
        @permission_required(Mock())
        @access_by_role
        def func():
            pass
        self.assertIs(getattr(func, 'access_by_role', False), True)

    def test_preserve_attribute_decorating_class_method(self):

        class DummyClass(object):
            @access_by_role
            def method(self):
                pass
        self.assertIs(getattr(DummyClass().method, 'access_by_role', False),
                      True)


class UnitTestAccessByRoleDecorator(unittest.TestCase):

    @patch('django_roles_access.decorator.check_access_by_role')
    def test_decorator_call_check_access_by_role(
            self, mock_check_access_by_role
    ):
        func = Mock()
        func.__name__ = 'func'  # Needed by Python 2.7
        request = Mock()
        decorated_func = access_by_role(func)
        decorated_func(request)
        assert mock_check_access_by_role.called

    @patch('django_roles_access.decorator.check_access_by_role')
    def test_decorator_call_check_access_by_role_with_request(
            self, mock_check_access_by_role
    ):
        func = Mock()
        func.__name__ = 'func'  # Needed by Python 2.7
        request = Mock()
        decorated_func = access_by_role(func)
        decorated_func(request)
        mock_check_access_by_role.assert_called_once_with(request)

    @patch('django_roles_access.decorator.check_access_by_role')
    def test_decorator_return_view_if_access_by_role_return_true(
            self, mock_check_access_by_role
    ):
        func = Mock()
        func.__name__ = 'func'  # Needed by Python 2.7
        request = Mock()
        decorated_func = access_by_role(func)
        mock_check_access_by_role.return_value = True
        response = decorated_func(request)
        assert response == func(request)

    @patch('django_roles_access.decorator.get_no_access_response')
    @patch('django_roles_access.decorator.check_access_by_role')
    def test_call_get_no_access_response_when_access_by_role_return_false(
            self, mock_check_access_by_role, mock_get_no_access_response
    ):
        func = Mock()
        func.__name__ = 'func'  # Needed by Python 2.7
        request = Mock()
        decorated_func = access_by_role(func)
        mock_check_access_by_role.return_value = False
        decorated_func(request)
        assert mock_get_no_access_response.called

    @patch('django_roles_access.decorator.get_no_access_response')
    @patch('django_roles_access.decorator.check_access_by_role')
    def test_call_once_get_no_access_response_when_access_by_role_return_false(
            self, mock_check_access_by_role, mock_get_no_access_response
    ):
        func = Mock()
        func.__name__ = 'func'  # Needed by Python 2.7
        request = Mock()
        decorated_func = access_by_role(func)
        mock_check_access_by_role.return_value = False
        decorated_func(request)
        assert mock_get_no_access_response.call_count == 1


class IntegratedTestAccessByRoleDecorator(TestCase):
    """
    Integrated Test using default behavior for SECURED applications.
    """

    def setUp(self):
        settings.__setattr__('SECURED', ['django_roles'])
        # User
        self.u1, created = User.objects.get_or_create(username='test-1')

    def tearDown(self):
        settings.__delattr__('SECURED')

    def test_get_200_status_with_view_function(self):
        self.client.force_login(self.u1)
        response = self.client.get(
            '/role-included2/view_by_role/')
        self.assertEqual(response.status_code, 200)

    def test_get_403_status_with_view_function(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/view_by_role/')
        self.assertEqual(response.status_code, 403)

    def test_get_200_status_with_class_view(self):
        self.client.force_login(self.u1)
        response = self.client.get(
            '/role-included2/view_by_role_class/')
        self.assertEqual(response.status_code, 200)

    def test_get_403_status_with_class_view(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/view_by_role_class/')
        self.assertEqual(response.status_code, 403)

    def test_get_200_status_with_direct_view(self):
        """
        A view without URLConf application
        """
        ViewAccess.objects.create(
            view='direct_access_view',
            type='au'
        )
        self.client.force_login(self.u1)
        response = self.client.get(
            '/direct_access_view/')
        self.assertEqual(response.status_code, 200)

    def test_get_403_status_with_direct_view(self):
        ViewAccess.objects.create(
            view='direct_access_view',
            type='au'
        )
        self.client.logout()
        response = self.client.get(
            '/direct_access_view/')
        self.assertEqual(response.status_code, 403)

    def test_default_behavior_when_no_configuration(self):
        """
        Default behavior is to remain public if Django Roles si installed but
        no more configuration or decorator is used.
        """
        self.client.logout()
        response = self.client.get(
            '/direct_view/')
        self.assertEqual(response.status_code, 200)

    def test_forbidden_behavior_without_configuration(self):
        self.client.logout()
        response = self.client.get(
            '/role-included2/view_by_role/')
        self.assertIn(DEFAULT_FORBIDDEN_MESSAGE, response.content.decode(
            'utf-8'))

    def test_forbidden_behavior_with_configuration(self):
        expected = '<h1>No access</h1><p>Contact administrator</p>'
        settings.__setattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE',
                             expected)
        self.client.logout()
        response = self.client.get(
            '/role-included2/view_by_role/')
        settings.__delattr__('DJANGO_ROLES_ACCESS_FORBIDDEN_MESSAGE')
        self.assertIn(expected, response.content.decode('utf-8'))

    def test_redirect_if_configured(self):
        settings.__setattr__('DJANGO_ROLES_ACCESS_REDIRECT', True)
        self.client.logout()
        response = self.client.get(
            '/role-included2/view_by_role/')
        settings.__delattr__('DJANGO_ROLES_ACCESS_REDIRECT')
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, settings.LOGIN_URL)



from django.conf.urls import url

with_path = False  # Testing for Django 1
try:
    from django.urls import include
    from django.urls import path, re_path
    with_path = True  # Testing for Django 2
except:
    from django.conf.urls import include

from . import views

if with_path:
    urlpatterns = [
        path('direct_access_view/', views.protected_view_by_role,
             name='direct_access_view'),
        path('direct_view/', views.direct_view, name='direct_view'),
        re_path(r'^role-included[135]/',
                include('tests.include_roles_url')),
        path('role-included2/',
             include('tests.include_roles_url', namespace='app-ns2')),
        path('nest1/', include('tests.include_nested_namespaces',
                               namespace='nest1_namespace')),
    ]
else:
    urlpatterns = [
        url(r'^direct_access_view/$', views.protected_view_by_role,
            name='direct_access_view'),
        url(r'^direct_view/$', views.direct_view,
            name='direct_view'),
        url(r'^role-included[135]/',
            include('tests.include_roles_url')),
        url(r'^role-included2/',
            include('tests.include_roles_url', namespace='app-ns2')),
        url(r'^nest1/', include('tests.include_nested_namespaces',
                                namespace='nest1_namespace')),
    ]



"""
This views are used for testing.
"""
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views import View

from django_roles_access.decorator import access_by_role
from django_roles_access.mixin import RolesMixin


def direct_view(request):
    """
    A simple view for testing default behavior.
    """
    return HttpResponse('THis is a simple view')


@access_by_role
def protected_view_by_role(request):
    """
    A simple view for testing access_by_role decorator.
    """
    user = request.user
    return HttpResponse('THis is a view processed by django-roles '
                        'middleware. Username is {{ user.username }}')


class ProtectedView(View):

    def get(self, request, *args, **kwargs):
        user = request.user
        return HttpResponse('THis is a view processed by django-roles '
                            'middleware. Username is {{ user.username }}')

    @method_decorator(access_by_role)
    def dispatch(self, request, *args, **kwargs):
        return super(ProtectedView, self).dispatch(request, *args, **kwargs)


class ProtectedMixinView(RolesMixin, View):

    def get(self, request, *args, **kwargs):
        user = request.user
        return HttpResponse('THis is a view processed by django-roles '
                            'middleware. Username is {{ user.username }}')


def middleware_view(request):
    """
    A simple view for testing django-roles middleware.
    """
    user = request.user
    return HttpResponse('THis is a view processed by django-roles '
                        'middleware. Username is {{ user.username }}')


class MiddlewareView(View):

    def get(self, request, *args, **kwargs):
        user = request.user
        return HttpResponse('THis is a view processed by django-roles '
                            'middleware. Username is {{ user.username }}')



from django.conf.urls import url
with_path = False  # Testing for Django 1
try:
    from django.urls import include
    from django.urls import path, re_path
    with_path = True  # Testing for Django 2
except:
    from django.conf.urls import include


app_name = 'roles-nested-namespace'

if with_path:
    urlpatterns = [
        path('nest2/', include('tests.include_roles_namespace',
                               namespace='nest2_namespace')),
    ]
else:
    urlpatterns = [
        url(r'^nest2/', include('tests.include_roles_namespace',
                                namespace='nest2_namespace')),
    ]



from django.conf.urls import url
with_path = False  # Testing for Django 1
try:
    from django.urls import include
    from django.urls import path, re_path
    with_path = True  # Testing for Django 2
except:
    from django.conf.urls import include

from . import views


app_name = 'roles-app-name'

if with_path:
    urlpatterns = [
        path('view_by_role/', views.protected_view_by_role,
             name='view_protected_by_role'),
        path('view_by_role_class/', views.ProtectedView.as_view(),
             name='class_view_protected_by_role'),
        path('mixin_class_view/', views.ProtectedMixinView.as_view(),
             name='mixin_class_view'),
        path('middleware_view_func/', views.middleware_view,
             name='middleware_view_func'),
        path('middleware_view_class/', views.MiddlewareView.as_view(),
             name='middleware_view_class'),
    ]
else:
    urlpatterns = [
        url(r'^view_by_role/$', views.protected_view_by_role,
            name='view_protected_by_role'),
        url(r'^view_by_role_class/$', views.ProtectedView.as_view(),
            name='class_view_protected_by_role'),
        url(r'^mixin_class_view/$', views.ProtectedMixinView.as_view(),
            name='mixin_class_view'),
        url(r'^middleware_view_func/$', views.middleware_view,
            name='middleware_view_func'),
        url(r'^middleware_view_class/$', views.MiddlewareView.as_view(),
            name='middleware_view_class'),
    ]
